VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cEventLogs"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : EventLogs.cls
' By        : L.J. Johnson       Date: 04-28-2001
' Comments  : Wrapper for API to read event log entries
' ---------------------------------------------
Option Explicit

' -------------------------------------------------
' Type of item to look up in the registry
' -------------------------------------------------
Private Const EventMessage             As Long = 0&
Private Const ParameterMessage         As Long = 1&

' -------------------------------------------------
'
' -------------------------------------------------
Public Type FilterData
   EvtInfo                             As Boolean
   EvtWarning                          As Boolean
   EvtError                            As Boolean
   EvtSuccessAudit                     As Boolean
   EvtFailureAudit                     As Boolean
   SourceInclude                       As Boolean
   Source1                             As String
   Source2                             As String
   Source3                             As String
   CategoryInclude                     As Boolean
   Category1                           As String
   Category2                           As String
   Category3                           As String
   EventIdInclude                      As Boolean
   EventID                             As Long
   EventDateInclude                    As Boolean
   EventDateFrom                       As Date
   EventDateTo                         As Date
End Type
Private m_typEventFilter               As FilterData

' -------------------------------------------------
' Save the filter and filter type
' -------------------------------------------------
Private m_lngFilterType                As Long
Private m_vntFilter                    As Variant
Private m_blnEventReadLogForward       As Boolean
Private m_blnEventDataReturnHex        As Boolean

' -------------------------------------------------
' Share the module-level event log handle
' -------------------------------------------------
Private m_lngEventLogHwnd              As Long

' -------------------------------------------------
' Used to save the library names in a static array
'     in the ResourceString method
' -------------------------------------------------
Private Type Source_Lib_Type
   SourceName                          As String
   LibName                             As String
End Type

' -------------------------------------------------
' Not an API declaration -- this is the UDT that
'   stores the info on each event record
' -------------------------------------------------
Public Type EventRecord
   EventTimeWritten                    As Date
   EventTimeCreated                    As Date
   EventSourceName                     As String
   'EventUserName                       As String
   EventUserSID                        As String
   EventComputerName                   As String
   EventType                           As String
   EventDescription                    As String
   EventData                           As String
   EventDataText                       As String
   EventCategory                       As Long
   EventCategoryString                 As String
   EventRecordNum                      As Long
   EventID                             As Long
End Type
Private m_atypEventRecords()           As EventRecord
 
' --------------------------------------------
' This is the API type declaration
' --------------------------------------------
Public Type EVENTLOGRECORD
   Length                              As Long
   Reserved                            As Long
   RecordNumber                        As Long
   TimeGenerated                       As Long
   TimeWritten                         As Long
   EventID                             As Long
   EventType                           As Integer
   NumStrings                          As Integer
   EventCategory                       As Integer
   ReservedFlags                       As Integer
   ClosingRecordNumber                 As Long
   StringOffset                        As Long
   UserSidLength                       As Long
   UserSidOffset                       As Long
   DataLength                          As Long
   DataOffset                          As Long
End Type

' -------------------------------------------------
' Used as part of error string return
' -------------------------------------------------
Private Const EVENT_SOURCENAME         As String = "ReadEventLogs.EventLogs."
Private Const FORMAT_SOURCENAME        As String = "ReadEventLogs.FormatMsg."

' -------------------------------------------------
' For keeping track of our own error info
' -------------------------------------------------
Private m_lngErrNumber                 As Long
Private m_strErrSource                 As String
Private m_strErrDesc                   As String

' -------------------------------------------------
' Used to pass info within the class module
' -------------------------------------------------
Private m_strTypeEventLog              As String
Private m_lngCount                     As Long

' -------------------------------------------------
' Used to get time/date info from event records
' -------------------------------------------------
Private Type SYSTEMTIME
   wYear                               As Integer
   wMonth                              As Integer
   wDayOfWeek                          As Integer
   wDay                                As Integer
   wHour                               As Integer
   wMinute                             As Integer
   wSecond                             As Integer
   wMilliseconds                       As Integer
End Type

Private Type TIME_ZONE_INFORMATION
   Bias                                As Long
   StandardName                        As String * 32
   StandardDate                        As SYSTEMTIME
   StandardBias                        As Long
   DaylightName                        As String * 32
   DaylightDate                        As SYSTEMTIME
   DaylightBias                        As Long
End Type

' -------------------------------------------------
' Used with LoadLibraryEx
' -------------------------------------------------
Private Const DONT_RESOLVE_DLL_REFERENCES    As Long = 1&
Private Const LOAD_LIBRARY_AS_DATAFILE       As Long = 2&
Private Const LOAD_WITH_ALTERED_SEARCH_PATH  As Long = 8&

' -------------------------------------------------
'
' -------------------------------------------------
Private Const FORMAT_MESSAGE_IGNORE_INSERTS  As Long = &H200&
Private Const FORMAT_MESSAGE_FROM_HMODULE    As Long = &H800&

Public Enum enmEventType
   EVENTLOG_SUCCESS = &H0&
   EVENTLOG_ERROR_TYPE = &H1&
   EVENTLOG_WARNING_TYPE = &H2&
   EVENTLOG_INFORMATION_TYPE = &H4&
   EVENTLOG_AUDIT_SUCCESS = &H8&
   EVENTLOG_AUDIT_FAILURE = &H10&
End Enum

' -------------------------------------------------
' 32-bit API declares
' -------------------------------------------------
Private Declare Function OpenEventLog _
   Lib "advapi32" Alias "OpenEventLogA" _
   (ByVal lpUNCServerName As String, _
    ByVal lpEventSourceName As String) As Long
Private Declare Function CloseEventLog _
   Lib "advapi32" _
   (ByVal hEventLog As Long) As Long
Private Declare Function OpenBackupEventLog _
   Lib "advapi32" Alias "OpenBackupEventLogA" _
   (ByVal lpUNCServerName As String, _
    ByVal lpFileName As String) As Long
Private Declare Function GetNumberOfEventLogRecords _
   Lib "advapi32.dll" _
   (ByVal hEventLog As Long, _
    NumberOfRecords As Long) As Long
Private Declare Function ReadEventLog _
   Lib "advapi32.dll" Alias "ReadEventLogA" _
   (ByVal hEventLog As Long, _
    ByVal dwReadFlags As Long, _
    ByVal dwRecordOffset As Long, _
    lpBuffer As Any, _
    ByVal nNumberOfBytesToRead As Long, _
    pnBytesRead As Long, _
    pnMinNumberOfBytesNeeded As Long) As Long
Private Declare Function LoadLibraryEx _
   Lib "Kernel32" Alias "LoadLibraryExA" _
   (ByVal lpLibFileName As String, _
    ByVal hFile As Long, _
    ByVal dwFlags As Long) As Long
Private Declare Function LoadLibrary _
   Lib "Kernel32" Alias "LoadLibraryA" _
   (ByVal lpLibFileName As String) As Long
Private Declare Function FreeLibrary _
   Lib "Kernel32" _
   (ByVal hLibModule As Long) As Long
Private Declare Function GetTimeZoneInformation _
   Lib "Kernel32" _
   (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
Private Declare Function ExpandEnvironmentStrings _
   Lib "Kernel32" Alias "ExpandEnvironmentStringsA" _
   (ByVal lpSrc As String, _
    ByVal lpDst As String, _
    ByVal nSize As Long) As Long

' -------------------------------------------------
' Defines for the READ flags for Eventlogging
' -------------------------------------------------
Private Const EVENTLOG_SEQUENTIAL_READ As Long = &H1
Private Const EVENTLOG_SEEK_READ       As Long = &H2
Private Const EVENTLOG_FORWARDS_READ   As Long = &H4
Private Const EVENTLOG_BACKWARDS_READ  As Long = &H8

Public Property Let EventFilter(ByRef xi_typEventFilter As FilterData)
   m_typEventFilter = xi_typEventFilter
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Sub OpenAnyEventLog
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:30:03
' Inputs       : ByVal xi_strServerName:String -
' Outputs      : N/A
' Description  : This method must be called before the
'              :     ReadEventEntries method.
'              : It opens the event log type (previously set via
'              :     the EventTypeLog property) on the NT computer
'              :     passed to the method
' *******************************************************
Public Sub OpenAnyEventLog(ByVal xi_strServerName As String, _
                           Optional ByVal xi_strFileName As String = "")
On Error Resume Next                   ' Raise your own errors
   Dim p_lngEventLogHwnd               As Long

   ' ----------------------------------------------
   ' Make sure that the Computer Name is trimmed
   '   and in UNC format
   ' ----------------------------------------------
   xi_strServerName = Trim$(xi_strServerName)
   
   If Len(xi_strServerName) = 0 Then
      ' -------------------------------------------
      ' Do nothing -- can't put '\\' in front of
      '   a blank string
      ' -------------------------------------------
   
   ElseIf Left$(xi_strServerName, 2) = "\\" Then
      ' -------------------------------------------
      ' Do nothing -- already has '\\' in front
      '   of server of UNC convention
      ' -------------------------------------------
   
   ElseIf Left$(xi_strServerName, 1) = "\" Then
      xi_strServerName = "\" & xi_strServerName
   
   Else
      xi_strServerName = "\\" & xi_strServerName
   End If

   ' ----------------------------------------------
   ' See if have an invalid EventLog Type
   ' ----------------------------------------------
   If Len(m_strTypeEventLog) = 0 Or _
      CStr(m_strTypeEventLog) = CStr(&HFFFF) Then
      m_lngEventLogHwnd = 0
      
      m_lngErrNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      m_strErrSource = EVENT_SOURCENAME & "OpenAnyEventLog"
      m_strErrDesc = "Error opening the event log: " & _
                     "EventType not has not been set."
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End If

   ' ----------------------------------------------
   ' Open the event log
   ' ----------------------------------------------
   If Len(Trim$(xi_strFileName)) > 0 Then
      p_lngEventLogHwnd = OpenBackupEventLog(xi_strServerName, xi_strFileName)
   Else
      p_lngEventLogHwnd = OpenEventLog(xi_strServerName, m_strTypeEventLog)
   End If
   
   ' ----------------------------------------------
   ' Check for any errors
   ' ----------------------------------------------
   If p_lngEventLogHwnd = 0 Then
      m_lngEventLogHwnd = 0
      
      m_lngErrNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      m_strErrSource = EVENT_SOURCENAME & "OpenAnyEventLog"
      m_strErrDesc = "Error opening the event log, " & _
                     m_strTypeEventLog & ", on the Server, " & xi_strServerName & _
                     ": " & ReturnApiErrString(Err.LastDllError)
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc

   Else
      m_lngEventLogHwnd = p_lngEventLogHwnd
   End If

End Sub

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Sub ReadEventEntries
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:32:03
' Inputs       : Optional ByVal xi_blnFilterEvents:Boolean = False -- Whether to filter events in the DLL or not
'              : Optional ByVal xi_lngNumRecsToRead:Long = -1      -- Number of records to read
' Outputs      : Long: Number of actual records returned
' Description  : This is the heart of the OLE server.  It reads
'              :     the selected event log on the selected NT
'              :     server, and makes calls to all the private
'              :     properties and functions
' *******************************************************
Public Function ReadEventEntries(Optional ByVal xi_blnFilterEvents As Boolean = False, _
                                 Optional ByVal xi_lngNumRecsToRead As Long = -1) As Long
On Error Resume Next                   ' Raise your own errors
   Dim p_lngLoop                       As Long
   Dim p_lngCount                      As Long
   Dim p_abytBuffer()                  As Byte     ' Used to read the event log records
   Dim p_lngEventLogHwnd               As Long     ' Handle to the current open event log
   Dim p_lngReadFlags                  As Long     ' Flags used to read the event log
   Dim p_lngNumBytesToRead             As Long     ' Number of bytes TO read from event log
   Dim p_lngNumBytesRead               As Long     ' Number of bytes actually READ from event log
   Dim p_lngNumRecords                 As Long     ' Total number of records in this event log
   Dim p_lngMinBytesNeeded             As Long     ' Used in ReadEventLog API call
   Dim p_lngRtn                        As Long     ' Return value from ReadEventLog API call
   Dim p_typEventRecAPI                As EVENTLOGRECORD
   Dim p_typEventRec                   As EventRecord
   
   ' ----------------------------------------------
   ' Make sure that the event log has been opened
   ' ----------------------------------------------
   If m_lngEventLogHwnd <> 0 Then
      p_lngEventLogHwnd = m_lngEventLogHwnd
   
   ElseIf m_lngEventLogHwnd = &HFFFF Then
      ' -------------------------------------------
      ' Has never been set -- see Initialize event
      ' -------------------------------------------
      m_lngErrNumber = (ERR_LOG_NOT_OPENED + vbObjectError)
      m_strErrSource = EVENT_SOURCENAME & "ReadEventEntries"
      m_strErrDesc = "The Event Log has not been opened yet.  " & _
                     "Use the OpenAnyEventLog property."
      m_lngCount = 0
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
      
   Else
      ' -------------------------------------------
      ' Error info set when set the Server Name
      ' -------------------------------------------
      m_lngCount = 0
      m_lngErrNumber = (ERR_BAD_SERVER_NAME + vbObjectError)
      m_strErrSource = EVENT_SOURCENAME & "ReadEventEntries"
      m_strErrDesc = "The Server name you entered " & _
                     "may not be a valid server name."
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End If

   ' ----------------------------------------------
   ' See how many event records are in this log
   ' ----------------------------------------------
   If API_NumberOfEventLogRecords(p_lngEventLogHwnd) = True Then
      p_lngNumRecords = CountEventRecords
   End If
   
   If p_lngNumRecords <= 0 Then
      ' -------------------------------------------
      ' No records -- set number of records to zero
      '   and exit the function (not necessarily an
      '   error)
      ' -------------------------------------------
      m_lngCount = 0
      GoTo Cleanup
      
   Else
      ' -------------------------------------------
      ' Redimension the type array to total number
      '   of records
      ' -------------------------------------------
      ReDim m_atypEventRecords(1 To p_lngNumRecords) As EventRecord
   
   End If

   ' ----------------------------------------------
   ' Set the read flags
   ' ----------------------------------------------
   If m_blnEventReadLogForward = True Then
      p_lngReadFlags = EVENTLOG_SEQUENTIAL_READ Or _
                       EVENTLOG_FORWARDS_READ
   Else
      p_lngReadFlags = EVENTLOG_SEQUENTIAL_READ Or _
                       EVENTLOG_BACKWARDS_READ
   End If

   ' ----------------------------------------------
   ' Initialize some variables
   ' ----------------------------------------------
   For p_lngLoop = 1 To p_lngNumRecords
   
      ' ---------------------------------------
      ' Bail out if only want a fixed number
      '     of records
      ' ---------------------------------------
      If xi_lngNumRecsToRead > 0 Then
         If p_lngLoop > xi_lngNumRecsToRead Then
            Exit For
         End If
      End If
      
      p_lngRtn = ReadEventLog(p_lngEventLogHwnd, _
                              p_lngReadFlags, _
                              p_lngLoop, _
                              p_typEventRecAPI, _
                              p_lngNumBytesToRead, _
                              p_lngNumBytesRead, _
                              p_lngMinBytesNeeded)
      If p_lngRtn = 0 And Err.LastDllError <> ERROR_INSUFFICIENT_BUFFER Then
         Debug.Print modMain.ReturnApiErrString(Err.LastDllError)
      ElseIf p_lngRtn = 0 And _
         Err.LastDllError = ERROR_INSUFFICIENT_BUFFER Then
     
         ' ------------------------------------
         ' DWORD align p_lngMinBytesNeeded
         ' ------------------------------------
         p_lngMinBytesNeeded = ((p_lngMinBytesNeeded + 3) \ 4) * 4
         ReDim p_abytBuffer(p_lngMinBytesNeeded - 1) As Byte
         
         If ReadEventLog(p_lngEventLogHwnd, _
                         p_lngReadFlags, _
                         p_lngLoop, _
                         p_abytBuffer(0), _
                         p_lngMinBytesNeeded, _
                         p_lngNumBytesRead, _
                         p_lngMinBytesNeeded) = 0 Then
            m_lngErrNumber = (ERR_READ_EVENT_RECORD + vbObjectError)
            m_strErrSource = EVENT_SOURCENAME & "ReadEventEntries"
            m_strErrDesc = "Could not read the EventLog record." & vbCrLf & _
                           "Error: " & modMain.ReturnApiErrString(Err.LastDllError)
            CloseEventLog p_lngEventLogHwnd
            On Error GoTo 0
            Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
         Else
         
            ' ---------------------------------
            ' Move to the structure
            ' ---------------------------------
            MoveMem pTo:=p_typEventRecAPI, _
                    uFrom:=p_abytBuffer(0), _
                    lSize:=Len(p_typEventRecAPI)
                        
            ' ---------------------------------
            ' Get the data for this record and
            '     add to collection *IF* the
            '     data matches the filter
            ' ---------------------------------
            Err.Clear
            If GetRecordData(xi_lngLoop:=p_lngLoop, _
                             xi_abytBuffer:=p_abytBuffer, _
                             xi_typEventRecAPI:=p_typEventRecAPI, _
                             xi_blnFilterEvents:=xi_blnFilterEvents, _
                             xio_typEventRec:=p_typEventRec) = True Then
               p_lngCount = p_lngCount + 1
               m_atypEventRecords(p_lngCount) = p_typEventRec
            Else
               ' Don't add the record
            End If
            If Err.Number <> 0 Then
               ' Don't raise an error here, just
               '     keep on trucking...
            End If
            
         End If
      End If
   
   Next p_lngLoop

   ' ------------------------------------------
   ' Cleanup the unused values in array and
   '     set return value
   ' ------------------------------------------
   If p_lngCount > 0 Then
      ReDim Preserve m_atypEventRecords(1 To p_lngCount) As EventRecord
   Else
      Erase m_atypEventRecords
   End If
   
   ReadEventEntries = p_lngCount
   
Cleanup:
   CloseEventLog p_lngEventLogHwnd
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Sub GetRecordData
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 13:48:58
' Inputs       : ByVal xi_lngLoop:Long                  -
'              : ByRef xi_abytBuffer():Byte             -
'              : ByRef xi_typEventRecAPI:EVENTLOGRECORD -
'              : ByRef xio_typEventRec:EventRecord      -
' Outputs      : N/A
' Description  : Get the event log record for exactly
'              :     one record
' *******************************************************
Private Function GetRecordData(ByVal xi_lngLoop As Long, _
                               ByRef xi_abytBuffer() As Byte, _
                               ByRef xi_typEventRecAPI As EVENTLOGRECORD, _
                               ByRef xio_typEventRec As EventRecord, _
                               Optional ByVal xi_blnFilterEvents As Boolean = False) As Boolean
   Dim p_lngRecNumber                  As Long
   Dim p_dteTimeGenerated              As Date
   Dim p_dteTimeWritten                As Date
   Dim p_lngEventID                    As Long
   Dim p_lngEventIdLong                As Long
   Dim p_lngEventType                  As Long
   Dim p_lngNumStrings                 As Long
   Dim p_lngEventCategory              As Long
   Dim p_lngSID                        As Long
   Dim p_lngLenLogRec                  As Long
   Dim p_lngStringLen                  As Long
   Dim p_lngCount                      As Long
   Dim p_blnShowRec                    As Boolean
   Dim p_strSourceName                 As String
   Dim p_strSID                        As String
   Dim p_strUserID                     As String
   Dim p_strComputerName               As String
   Dim p_strEventType                  As String
   Dim p_strStrings                    As String
   Dim p_strMsg                        As String
   Dim p_strData                       As String
   Dim p_strDataText                   As String
   Dim p_strEventCategoryName          As String
   Dim p_abytData()                    As Byte

   p_lngLenLogRec = Len(xi_typEventRecAPI)
   
   With xi_typEventRecAPI
   
      ' ---------------------------------
      ' Get the easy stuff
      ' ---------------------------------
      p_lngRecNumber = .RecordNumber
      p_dteTimeGenerated = DateToLocalDate(DateAdd("s", .TimeGenerated, #1/1/1970#))
      p_dteTimeWritten = DateToLocalDate(DateAdd("s", .TimeWritten, #1/1/1970#))
      p_lngEventID = modMain.LoWord(.EventID)
      p_lngEventIdLong = .EventID
      p_lngEventType = .EventType
      'Debug.Print "Event Type: " & p_lngEventType
      p_lngNumStrings = .NumStrings
      p_lngEventCategory = .EventCategory
      
      ' ---------------------------------
      ' Source is the first text after the structure
      ' ---------------------------------
      p_strSourceName = GetStrFromPtrA(VarPtr(xi_abytBuffer(p_lngLenLogRec)))
      
      ' ---------------------------------------
      ' Only do this if have a non-zero
      '     Event Category
      ' ---------------------------------------
      If p_lngEventCategory <> 0 Then
         p_strEventCategoryName = CategoryString(p_strSourceName, _
                                                 p_lngEventCategory)
         p_strEventCategoryName = "(" & CStr(p_lngEventCategory) & _
                                  "): " & p_strEventCategoryName
      Else
         p_strEventCategoryName = "None"
      End If
      
      ' ---------------------------------
      ' Get the text string for Event Type
      ' ---------------------------------
      p_strEventType = GetEventTypeText(xi_lngEventType:=p_lngEventType)
      
      ' ---------------------------------------
      ' Fill in enough info to determine if
      '     we'll save this record
      ' ---------------------------------------
      With xio_typEventRec
         .EventRecordNum = p_lngRecNumber
         .EventTimeCreated = p_dteTimeGenerated
         .EventTimeWritten = p_dteTimeWritten
         .EventID = p_lngEventID
         .EventType = p_strEventType
         .EventSourceName = p_strSourceName
         .EventCategory = p_lngEventCategory
      End With
      
      If (xi_blnFilterEvents = True) Then
         p_blnShowRec = modFilterEvents.FilterRecord(xi_typRecord:=xio_typEventRec, _
                                                     xi_typUserFilterData:=m_typEventFilter)
      Else
         p_blnShowRec = True
      End If
      
      ' ---------------------------------------
      ' This data is more "expensive" -- no need
      '     to get if we aren't going to show it
      ' ---------------------------------------
      If p_blnShowRec = True Then
         
         ' ---------------------------------
         ' Computer name is next section
         ' ---------------------------------
         p_strComputerName = GetStrFromPtrA(VarPtr(xi_abytBuffer(p_lngLenLogRec + _
                                            Len(p_strSourceName) + 1)))
         
         ' ---------------------------------
         ' SID is next
         ' ---------------------------------
         If .UserSidLength > 0 Then
            p_lngSID = VarPtr(xi_abytBuffer(.UserSidOffset))
            p_strSID = GetTextualSid(p_lngSID)
            'p_strUserID = GetUserNameSID(p_lngSID)
         Else
            p_strSID = "<none>"
         End If
            
         ' ---------------------------------
         ' Strings follow -- need to get
         '     the length here, otherwise
         '     function will truncate at
         '     first null
         ' ---------------------------------
         p_lngStringLen = (.DataOffset - _
                           .StringOffset) _
                           + 1
         p_strStrings = GetStrFromPtrA(VarPtr(xi_abytBuffer(.StringOffset)), _
                                       p_lngStringLen)
         p_strMsg = ResourceString(p_strSourceName, _
                                   p_lngNumStrings, _
                                   p_strStrings, _
                                   p_lngEventIdLong, _
                                   p_lngEventID)
         
         ' ---------------------------------
         ' Get the data -- ditto for NULLs
         ' ---------------------------------
         p_strData = vbNullString
         p_lngStringLen = (.DataLength)
         If .DataLength > 0 Then
            p_abytData = GetStrFromPtrA(VarPtr(xi_abytBuffer(.DataOffset)), _
                                        p_lngStringLen)
            p_strData = ConvertToHex(xi_lngDataLength:=.DataLength, _
                                     xi_abytData:=p_abytData, _
                                     xio_strText:=p_strDataText)
         Else
            p_strData = "N/A"
         End If
         
      End If
      
   End With
   
   ' ---------------------------------
   ' Fill in the collection for this
   '     record
   ' ---------------------------------
   With xio_typEventRec
      .EventCategoryString = p_strEventCategoryName
      .EventComputerName = p_strComputerName
      .EventUserSID = p_strSID
      .EventDescription = p_strMsg
      .EventData = p_strData
      .EventDataText = p_strDataText
      '.EventUserName = p_strUserID
   End With
   
   ' ------------------------------------------
   '
   ' ------------------------------------------
   GetRecordData = p_blnShowRec
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function GetEventTypeText
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 13:15:33
' Inputs       : ByVal xi_lngEventType:Long -
' Outputs      : STRING:
' Description  : Return the string for the numeric
'              :     Event Type
' *******************************************************
Private Function GetEventTypeText(ByVal xi_lngEventType As Long) As String

   Select Case CLng(xi_lngEventType)
      Case EVENTLOG_SUCCESS
         GetEventTypeText = "Success"
      Case EVENTLOG_ERROR_TYPE
         GetEventTypeText = "Error"
      Case EVENTLOG_WARNING_TYPE
         GetEventTypeText = "Warning"
      Case EVENTLOG_INFORMATION_TYPE
         GetEventTypeText = "Information"
      Case EVENTLOG_AUDIT_SUCCESS
         GetEventTypeText = "Audit Success"
      Case EVENTLOG_AUDIT_FAILURE
         GetEventTypeText = "Audit Failure"
   End Select

End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function ConvertToHex
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 13:50:24
' Inputs       : ByVal xi_lngDataLength:Long -
'              : ByRef xi_abytData():Byte -
' Outputs      : STRING:
' Description  : Convert to hex only if less than
'              :     256 bytes or if the user wants
'              :     it that way
' *******************************************************
Private Function ConvertToHex(ByVal xi_lngDataLength As Long, _
                              ByRef xi_abytData() As Byte, _
                              ByRef xio_strText As String) As String
   Dim p_lngLoop                       As Long
   Dim p_strData                       As String
   Dim p_strText                       As String
   
   If m_blnEventDataReturnHex = False Then
      If xi_lngDataLength > 256 Then
         p_strData = xi_abytData
      Else
         For p_lngLoop = 0 To UBound(xi_abytData)
            p_strData = p_strData & Format$(Hex$(xi_abytData(p_lngLoop)), "00") & " "
         Next p_lngLoop
      
         xio_strText = xi_abytData
         xio_strText = Replace(xio_strText, Chr$(0), "")
      End If
   
   Else
      For p_lngLoop = 0 To UBound(xi_abytData)
         p_strData = p_strData & Format$(Hex$(xi_abytData(p_lngLoop)), "00") & " "
      Next p_lngLoop
      
      xio_strText = xi_abytData
      xio_strText = Replace(xio_strText, Chr$(0), "")
   End If

   ConvertToHex = p_strData
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function API_NumberOfEventLogRecords
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:52:27
' Inputs       : ByVal xi_lngEventLogHwnd:Long -
' Outputs      : N/A
' Description  : Get the number of records in this event log using
'              :     the API -- note that this is not exposed by
'              :     the object, since the "actual" number may
'              :     changed based on the filtering applied
' *******************************************************
Private Function API_NumberOfEventLogRecords(ByVal xi_lngEventLogHwnd As Long) As Long
   Dim p_lngNumRecords                 As Long
   
   If GetNumberOfEventLogRecords(xi_lngEventLogHwnd, _
                  p_lngNumRecords) <> False Then
      m_lngCount = p_lngNumRecords
      API_NumberOfEventLogRecords = True
   Else
      API_NumberOfEventLogRecords = False
   End If

End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function CategoryString
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/31/2001 -- 12:23:11
' Inputs       : ByVal xi_strEvtSourceName:String  -
'              : ByVal xi_lngCategoryID:Long       -
' Outputs      : N/A
' Description  : Given the category number, return the
'              :     category string
' *******************************************************
Private Function CategoryString(ByVal xi_strEvtSourceName As String, _
                                ByVal xi_lngCategoryID As Long) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_strApiError                   As String
   Dim p_strLibName                    As String
   Dim p_strMsgString                  As String
   Dim p_strNewLibName                 As String
   Dim p_strSource                     As String
   Dim p_lngNumEventIDs                As Long
   Dim p_lngNumLibNames                As Long
   Dim p_lngRtn                        As Long
   Dim p_lngRtnErr                     As Long
   Dim p_lngPos                        As Long
   Dim p_lngPos2                       As Long
   Dim p_lngModHwnd                    As Long
   Dim p_lngLoop                       As Long
   Dim p_lngFoundLib                   As Long
   Dim p_lngTransRtn                   As Long
   Dim p_astrMsgArg()                  As String
   Dim p_objRegistryDB                 As New cRegistryDB
   Dim p_typCategories                 As FmtMsgArrayType
   
   Static p_statypLibNames()           As Source_Lib_Type

   ' ----------------------------------------------
   ' Exit if passed a blank SourceName
   ' ----------------------------------------------
   If Len(Trim$(xi_strEvtSourceName)) <= 0 Then
      CategoryString = vbNullString
      Exit Function
   End If

   ' ----------------------------------------------
   ' Get the size of the p_statypLibNames array
   ' ----------------------------------------------
   p_lngNumLibNames = 0
   p_lngNumLibNames = UBound(p_statypLibNames)
   
   ' ----------------------------------------------
   ' See if we already have the libray name in the
   '   static array
   ' ----------------------------------------------
   If p_lngNumLibNames > 0 Then
      p_lngFoundLib = False
      For p_lngLoop = 1 To p_lngNumLibNames
         If p_statypLibNames(p_lngLoop).SourceName = xi_strEvtSourceName Then
            p_strLibName = p_statypLibNames(p_lngLoop).LibName
            p_lngFoundLib = True
            Exit For
         End If
      Next p_lngLoop
   Else
      p_lngFoundLib = False
   End If

   ' ----------------------------------------------
   ' Didn't find the libary name in array,
   '   so get it with the API
   ' ----------------------------------------------
   If p_lngFoundLib = False Then
      
      ' -------------------------------------------
      ' Get info from the registry
      ' -------------------------------------------
      p_objRegistryDB.LogType = m_strTypeEventLog
      p_objRegistryDB.SourceName = xi_strEvtSourceName
      
      p_objRegistryDB.RegistryEntry xi_enmMsgType:=CategoryMessage
      If Err.Number = 0 Then
         p_strLibName = p_objRegistryDB.RegistryEntryText
      Else
         ' ------------------------------------
         ' Failed to find the entry in registry,
         '     so just return a blank string
         ' ------------------------------------
         p_strMsgString = Err.Description
         CategoryString = vbNullString
         Exit Function
      End If

      ' -------------------------------------------
      ' Expand any environmental strings in the
      '     registry entry
      ' -------------------------------------------
      p_strLibName = UCase$(p_strLibName)
      p_lngPos = InStr(p_strLibName, "%")
      If p_lngPos > 0 Then
         p_strNewLibName = Space$(256)
         p_lngRtn = ExpandEnvironmentStrings(lpSrc:=p_strLibName, _
                                             lpDst:=p_strNewLibName, _
                                             nSize:=Len(p_strNewLibName))
         If p_lngRtn > 0 Then
            p_strLibName = Mid$(p_strNewLibName, 1, p_lngRtn)
         End If
      End If
      
      ' -------------------------------------------
      ' Add to static array
      ' -------------------------------------------
      p_lngNumLibNames = p_lngNumLibNames + 1
      ReDim Preserve p_statypLibNames(1 To p_lngNumLibNames)
      p_statypLibNames(p_lngNumLibNames).SourceName = xi_strEvtSourceName
      p_statypLibNames(p_lngNumLibNames).LibName = p_strLibName
   End If

   ' ----------------------------------------------
   ' Load the module
   ' Note: If you use LoadLibrary, it will fail
   '       on certain libraries. You will need to use
   '       the LOAD_LIBRARY_AS_DATAFILE flag.
   ' "Use this flag when you want to load a DLL only
   '       to extract messages or resources from it"
   ' ----------------------------------------------
   p_lngModHwnd = LoadLibraryEx(lpLibFileName:=p_strLibName, _
                                hFile:=0&, _
                                dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
   
   ' ----------------------------------------------
   ' Get the string from the module
   ' ----------------------------------------------
   If p_lngModHwnd > 0 Then
      p_lngRtnErr = FormatMsgFromResource(p_lngModHwnd, _
                                          xi_lngCategoryID, _
                                          p_typCategories, _
                                          p_strMsgString, _
                                          p_strApiError, _
                                          False)
      If p_lngRtnErr <> 0 Then
         m_lngErrNumber = p_lngRtnErr
         m_strErrSource = FORMAT_SOURCENAME & "FormatMsgFromResource"
         m_strErrDesc = p_strApiError
         
         p_strMsgString = m_strErrDesc
         CategoryString = vbNullString
         'Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
      Else
         ' ------------------------------------
         ' Got message string, don't need to
         '     do anything
         ' ------------------------------------
      End If
   Else
      p_strMsgString = vbNullString
   End If

   ' ----------------------------------------------
   ' Clean up the string
   ' ----------------------------------------------
   p_strMsgString = ClearLastCrLf(Trim$(p_strMsgString))

   ' ----------------------------------------------
   ' Free the module
   ' ----------------------------------------------
   FreeLibrary p_lngModHwnd

   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   CategoryString = p_strMsgString
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function ResourceString
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:28:59
' Inputs       : ByVal xi_strEvtSourceName:String -
'              : ByVal xi_lngNumStrings:Long -
'              : ByVal xi_strStrings:String -
'              : ByVal xi_lngEventID:Long -
'              : ByVal xi_lngEventIdForDisplay:Long -
' Outputs      : STRING:
' Description  : Private method to read the strings in
'              :     the resource file
' *******************************************************
Private Function ResourceString(ByVal xi_strEvtSourceName As String, _
                                ByVal xi_lngNumStrings As Long, _
                                ByVal xi_strStrings As String, _
                                ByVal xi_lngEventID As Long, _
                                ByVal xi_lngEventIdForDisplay As Long) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_strApiError                   As String
   Dim p_strLibName                    As String
   Dim p_strMsgString                  As String
   Dim p_strNewLibName                 As String
   Dim p_strSource                     As String
   Dim p_lngNumEventIDs                As Long
   Dim p_lngNumLibNames                As Long
   Dim p_lngRtn                        As Long
   Dim p_lngRtnErr                     As Long
   Dim p_lngPos                        As Long
   Dim p_lngPos2                       As Long
   Dim p_lngModHwnd                    As Long
   Dim p_lngLoop                       As Long
   Dim p_lngFoundLib                   As Long
   Dim p_lngTransRtn                   As Long
   Dim p_astrMsgArg()                  As String
   Dim p_objRegistryDB                 As New cRegistryDB
   Dim p_typMsgs                       As FmtMsgArrayType
   
   Static p_statypLibNames()           As Source_Lib_Type

   ' ----------------------------------------------
   ' Exit if passed a blank SourceName
   ' ----------------------------------------------
   If Len(Trim$(xi_strEvtSourceName)) <= 0 Then
      ResourceString = vbNullString
      Exit Function
   End If

   ' ----------------------------------------------
   ' Get the size of the p_statypLibNames array
   ' ----------------------------------------------
   p_lngNumLibNames = 0
   p_lngNumLibNames = UBound(p_statypLibNames)
   
   ' ----------------------------------------------
   ' See if we already have the libray name in the
   '   static array
   ' ----------------------------------------------
   If p_lngNumLibNames > 0 Then
      p_lngFoundLib = False
      For p_lngLoop = 1 To p_lngNumLibNames
         If p_statypLibNames(p_lngLoop).SourceName = xi_strEvtSourceName Then
            p_strLibName = p_statypLibNames(p_lngLoop).LibName
            p_lngFoundLib = True
            Exit For
         End If
      Next p_lngLoop
   Else
      p_lngFoundLib = False
   End If

   ' ----------------------------------------------
   ' Didn't find the libary name in array,
   '   so get it with the API
   ' ----------------------------------------------
   If p_lngFoundLib = False Then
      
      ' -------------------------------------------
      ' Get info from the registry
      ' -------------------------------------------
      p_objRegistryDB.LogType = m_strTypeEventLog
      p_objRegistryDB.SourceName = xi_strEvtSourceName
      
      p_objRegistryDB.RegistryEntry xi_enmMsgType:=EventMessage
      If Err.Number = 0 Then
         p_strLibName = p_objRegistryDB.RegistryEntryText
      Else
         ' ------------------------------------
         ' Failed to find the entry in registry,
         '     but rest of info may be OK
         ' Return the error message as part of
         '     the log item
         ' ------------------------------------
         p_strMsgString = Err.Description
         
         ' ----------------------------------------
         ' NOTE:  You may want to just return a
         '        blank string, but this is the
         '        same string as is present in the
         '        Event Viewer
         ' ----------------------------------------
         p_strMsgString = p_strMsgString & vbCrLf & _
                          "The description for Event ID (" & _
                          CStr(xi_lngEventIdForDisplay) & ") " & _
                          "in Source (" & Trim$(xi_strEvtSourceName) & _
                          ") could not be found. The local computer may not " & _
                          "have the necessary registry information or " & _
                          "message DLL files to display messages. The following " & _
                          "information is part of the event: " & vbCrLf & _
                          xi_strStrings
         ResourceString = p_strMsgString
         Exit Function
      End If

      ' -------------------------------------------
      ' Replace environmental string(s) if any
      ' -------------------------------------------
      p_strLibName = UCase$(p_strLibName)
      p_lngPos = InStr(p_strLibName, "%")
      If p_lngPos > 0 Then
         p_strNewLibName = Space$(256)
         p_lngRtn = ExpandEnvironmentStrings(p_strLibName, p_strNewLibName, Len(p_strNewLibName))
         If p_lngRtn > 0 Then
            p_strLibName = Mid$(p_strNewLibName, 1, p_lngRtn)
         End If
      End If

      ' -------------------------------------------
      ' Add to static array
      ' -------------------------------------------
      p_lngNumLibNames = p_lngNumLibNames + 1
      ReDim Preserve p_statypLibNames(1 To p_lngNumLibNames)
      p_statypLibNames(p_lngNumLibNames).SourceName = xi_strEvtSourceName
      p_statypLibNames(p_lngNumLibNames).LibName = p_strLibName
   End If

   ' ----------------------------------------------
   ' If have replacement strings, then parse
   '   them into an array to make them easier
   '   to deal with
   ' ----------------------------------------------
   If xi_lngNumStrings > 0 Then
      ReDim p_astrMsgArg(0 To xi_lngNumStrings - 1)
      
      p_astrMsgArg = Split(xi_strStrings, Chr$(0), -1)
      
      ' ---------------------------------------
      ' Gets more complicated -- we have some
      '   parameters to replace from the
      '   ParameterMessageFile instead of the
      '   EventMessageFile.
      ' ---------------------------------------
      For p_lngLoop = 0 To xi_lngNumStrings - 1
         If InStr(1, p_astrMsgArg(p_lngLoop), "%%") > 0 Then
            p_astrMsgArg(p_lngLoop) = ResourceString2(xi_strEvtSourceName, p_astrMsgArg(p_lngLoop))
         End If
      Next
      
      On Error GoTo 0
   
   End If

   ' ----------------------------------------------
   ' Get the string into a format which
   '   FormatMessage can use
   ' ----------------------------------------------
   If xi_lngNumStrings > 0 Then
      p_lngTransRtn = TranslateArray(p_astrMsgArg(), p_typMsgs)
   End If

   ' ----------------------------------------------
   ' You can have multiple files listed, separated
   '     by a semicolon. Get just the first item
   ' Actually, you should search the remaining
   '     files, in order, if you can't find the info
   '     in the first one, but I'm leaving that
   '     as an exercise to the reader.
   ' ----------------------------------------------
   p_lngPos = InStr(1, p_strLibName, ";")
   If p_lngPos > 0 Then
      p_strLibName = Mid$(p_strLibName, 1, p_lngPos - 1)
   End If
   
   ' ----------------------------------------------
   ' If path contains spaces, get the short name
   ' ----------------------------------------------
   If InStr(1, p_strLibName, " ") > 0 Then
      p_strLibName = modMain.GetShortName(p_strLibName)
   End If
   
   ' ----------------------------------------------
   ' Load the module
   ' Note: If you use LoadLibrary, it will fail
   '       on certain libraries. You will need to use
   '       the LOAD_LIBRARY_AS_DATAFILE flag.
   ' "Use this flag when you want to load a DLL only
   '       to extract messages or resources from it"
   ' ----------------------------------------------
   p_lngModHwnd = LoadLibraryEx(lpLibFileName:=p_strLibName, _
                                hFile:=0&, _
                                dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
   
   ' ----------------------------------------------
   ' Get the string from the module
   ' ----------------------------------------------
   If p_lngModHwnd > 0 Then
      p_lngRtnErr = FormatMsgFromResource(p_lngModHwnd, _
                                          xi_lngEventID, _
                                          p_typMsgs, _
                                          p_strMsgString, _
                                          p_strApiError, _
                                          True)
      If p_lngRtnErr <> 0 Then
         m_lngErrNumber = p_lngRtnErr
         m_strErrSource = FORMAT_SOURCENAME & "FormatMsgFromResource"
         m_strErrDesc = p_strApiError
      
         p_strMsgString = vbNullString
         ResourceString = vbNullString
         'Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
         p_strMsgString = m_strErrDesc
      End If
   Else
      p_strMsgString = "Error loading the library, " & vbCrLf & p_strLibName & vbCrLf & "Error: " & modMain.ReturnApiErrString(Err.LastDllError)
   End If

   ' ----------------------------------------------
   ' Clean up the string
   ' ----------------------------------------------
   p_strMsgString = ClearLastCrLf(Trim$(p_strMsgString))

   ' ----------------------------------------------
   ' Free the module
   ' ----------------------------------------------
   FreeLibrary p_lngModHwnd

   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ResourceString = p_strMsgString
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Function ResourceString2
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:24:54
' Inputs       : ByVal xi_strEventSourceName:String -
'              : ByVal xi_strMsgArg:String -
' Outputs      : STRING:
' Description  : Private function to translate the embedded "%%"
'              :     parameters from the resource file with the
'              :     items returned from parameter file
' *******************************************************
Private Function ResourceString2(ByVal xi_strEventSourceName As String, _
                                 ByVal xi_strMsgArg As String) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_objRegistryDB                 As New cRegistryDB
   Dim p_strLibName                    As String
   Dim p_strTmp                        As String
   Dim p_strParam                      As String
   Dim p_strReplaceable                As String
   Dim p_strNewLibName                 As String
   Dim p_astrTmpArray()                As String
   Dim p_lngPos                        As Long
   Dim p_lngItem                       As Long
   Dim p_lngFlags                      As Long
   Dim p_lngSize                       As Long
   Dim p_lngLoop                       As Long
   Dim p_lngModuleHwnd                 As Long
   Dim p_lngRtn                        As Long
   Dim p_lngNumItems                   As Long
   
   ' -------------------------------------------
   '
   ' -------------------------------------------
   p_strTmp = xi_strMsgArg

   ' -------------------------------------------
   ' Get info from the registry
   ' -------------------------------------------
   p_objRegistryDB.LogType = m_strTypeEventLog
   p_objRegistryDB.SourceName = xi_strEventSourceName
   
   p_objRegistryDB.RegistryEntry xi_enmMsgType:=ParameterMessage
   If Err.Number = 0 Then
      p_strLibName = p_objRegistryDB.RegistryEntryText
   
   Else
      ' Just pass on the erro
      ResourceString2 = vbNullString
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc

   End If

   ' -------------------------------------------
   ' Get the fully-replaced library name
   ' -------------------------------------------
   p_strLibName = UCase$(p_strLibName)
   p_lngPos = InStr(p_strLibName, "%")
   If p_lngPos > 0 Then
      p_strNewLibName = Space$(256)
      p_lngRtn = ExpandEnvironmentStrings(p_strLibName, _
                                          p_strNewLibName, _
                                          Len(p_strNewLibName))
      If p_lngRtn > 0 Then
         p_strLibName = Mid$(p_strNewLibName, 1, p_lngRtn)
      End If
   End If

   ' -------------------------------------------
   ' Load the library -- if fails, just return
   '     the original string
   ' -------------------------------------------
   p_lngModuleHwnd = LoadLibrary(p_strLibName)
   If p_lngModuleHwnd = 0 Then
      ResourceString2 = xi_strMsgArg
      Exit Function
   End If

   ' -------------------------------------------
   ' Kind of junky, but we need to find the
   '     number of "%%"'s in the string. Using
   '     SPLIT gives up an array of #items + 1,
   '     but it's zero-based, so UBOUND is the
   '     number of items
   ' -------------------------------------------
   p_astrTmpArray = Split(p_strTmp, "%%", -1)
   p_lngNumItems = UBound(p_astrTmpArray)
   
   ' -------------------------------------------
   ' Replace each parameter starting with %%
   '   (and get rid of CR-LF pairs)
   ' -------------------------------------------
   For p_lngLoop = 0 To p_lngNumItems - 1
      
      p_lngItem = 0
      p_lngItem = CLng(p_astrTmpArray(p_lngLoop + 1))
      
      p_strReplaceable = "%%" & CStr(p_lngItem)
      
      ' ----------------------------------------
      ' Set the flags for the API call
      ' ----------------------------------------
      p_lngFlags = FORMAT_MESSAGE_FROM_HMODULE Or _
                   FORMAT_MESSAGE_IGNORE_INSERTS
      p_lngSize = 512
      p_strParam = Space$(p_lngSize)

      ' ----------------------------------------
      ' Get the message string from the file
      ' ----------------------------------------
      p_lngRtn = FormatMessage(p_lngFlags, _
                               ByVal p_lngModuleHwnd, _
                               p_lngItem, _
                               &H0&, _
                               p_strParam, _
                               p_lngSize, _
                               ByVal 0&)

      ' ----------------------------------------
      ' Check for errors
      ' ----------------------------------------
      If p_lngRtn = 0 Then
         m_lngErrNumber = (ERR_RESOURCE_DATA_NOT_FOUND + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "ResourceString"
         m_strErrDesc = ReturnApiErrString(Err.LastDllError)
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc

      Else
         p_strParam = Trim$(Left$(p_strParam, p_lngRtn))
         
         If Right$(p_strParam, 2) = vbCrLf Then
            p_strParam = Left$(p_strParam, Len(p_strParam) - 2)
         End If
      End If
   
      p_strTmp = Replace(p_strTmp, p_strReplaceable, p_strParam)
   
   Next p_lngLoop
   
   ' -------------------------------------------
   ' Set the return value
   ' -------------------------------------------
   ResourceString2 = p_strTmp

   ' -------------------------------------------
   ' Reset error trapping
   ' -------------------------------------------
   On Error GoTo 0
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function GetNumRecords
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 14:29:08
' Inputs       : ByVal xi_strLogType:String -
' Outputs      : LONG:
' Description  : Get the number of records (at this instance)
'              :     for this log
' *******************************************************
Public Function GetNumRecords(ByVal xi_strLogType As String, _
                              ByVal xi_strServerName As String) As Long
On Error GoTo Cleanup
   Dim p_lngNumRecords                 As Long
   Dim p_lngRtn                        As Long
   
   m_strTypeEventLog = xi_strLogType
   OpenAnyEventLog xi_strServerName
   
   If GetNumberOfEventLogRecords(m_lngEventLogHwnd, _
                                 p_lngNumRecords) <> False Then
      GetNumRecords = p_lngNumRecords
   Else
      GetNumRecords = -1
   End If

Exit Function

Cleanup:
   m_lngErrNumber = Err.Number
   m_strErrSource = Err.Source
   m_strErrDesc = Err.Description
   
   CloseEventLog hEventLog:=m_lngEventLogHwnd
   
   Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   
End Function

' ***********************************************************************
' ***********************************************************************
' ****                     Read/Write Properties                     ****
' ***********************************************************************
' ***********************************************************************

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventTimeWritten
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:45:48
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : DATE:
' Description  : READ-ONLY: Return the Time Written for an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventTimeWritten(ByVal xi_lngIndex As Long) As Date
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventTimeWritten"
         m_strErrDesc = "The passed index number is not valid."
         EventTimeWritten = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventTimeWritten = m_atypEventRecords(xi_lngIndex).EventTimeWritten

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventTimeCreated
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:44:56
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : DATE:
' Description  : READ-ONLY: Return the Time Created an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventTimeCreated(ByVal xi_lngIndex As Long) As Date
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventTimeWritten"
         m_strErrDesc = "The passed index number is not valid."
         EventTimeCreated = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventTimeCreated = m_atypEventRecords(xi_lngIndex).EventTimeCreated

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventSourceName
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:44:01
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : STRING:
' Description  : READ-ONLY: Return the event Source Name of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventSourceName(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventSourceName"
         m_strErrDesc = "The passed index number is not valid."
         EventSourceName = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventSourceName = m_atypEventRecords(xi_lngIndex).EventSourceName

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventUserSID
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:43:05
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : STRING:
' Description  : READ-ONLY: Return the user's SID of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventUserSID(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventUserName"
         m_strErrDesc = "The passed index number is not valid."
         EventUserSID = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventUserSID = m_atypEventRecords(xi_lngIndex).EventUserSID

End Property

'' -------------------------------------------------
'' Return the Event User Name of an event log
''   record (based on passed index value)
'' Check for zero values or values greater than
''   the number of log entries in the filtered
''   result set.
'' -------------------------------------------------
'Public Property Get EventUserName(ByVal xi_lngIndex As Long) As String
'
'   Select Case xi_lngIndex
'      Case 0, Is > m_lngCount
'         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
'         m_strErrSource = EVENT_SOURCENAME & "EventUserName"
'         m_strErrDesc = "The passed index number is not valid."
'         EventUserName = vbNullString
'         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
'   End Select
'
'   EventUserName = m_atypEventRecords(xi_lngIndex).EventUserName
'
'End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventComputerName
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:41:58
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : STRING:
' Description  : READ-ONLY: Return the Event Computer Name of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventComputerName(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventComputerName"
         m_strErrDesc = "The passed index number is not valid."
         EventComputerName = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventComputerName = m_atypEventRecords(xi_lngIndex).EventComputerName

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventType
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:40:43
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : STRING:
' Description  : READ-ONLY:  Return the Event Type of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventType(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventType"
         m_strErrDesc = "The passed index number is not valid."
         EventType = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventType = m_atypEventRecords(xi_lngIndex).EventType

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventDescription
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:39:14
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : STRING
' Description  : READ-ONLY:  Return the Event Description of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventDescription(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventDescription"
         m_strErrDesc = "The passed index number is not valid."
         EventDescription = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventDescription = m_atypEventRecords(xi_lngIndex).EventDescription

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventCategoryString
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:38:05
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : LONG:
' Description  : READ-ONLY:  Return the Event Category of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventCategoryString(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventCategoryString"
         m_strErrDesc = "The passed index number is not valid."
         EventCategoryString = 0
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventCategoryString = m_atypEventRecords(xi_lngIndex).EventCategoryString

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventCategory
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:38:05
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : LONG:
' Description  : READ-ONLY:  Return the Event Category of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventCategory(ByVal xi_lngIndex As Long) As Long
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventCategory"
         m_strErrDesc = "The passed index number is not valid."
         EventCategory = 0
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventCategory = m_atypEventRecords(xi_lngIndex).EventCategory

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventRecordNum
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:37:14
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : N/A
' Description  : READ-ONLY:  Return the Event Record Number of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventRecordNum(ByVal xi_lngIndex As Long) As Long
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventRecordNum"
         m_strErrDesc = "The passed index number is not valid."
         EventRecordNum = 0
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventRecordNum = m_atypEventRecords(xi_lngIndex).EventRecordNum

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventID
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:36:04
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : LONG:
' Description  : READ-ONLY: Return the Event ID of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventID(ByVal xi_lngIndex As Long) As Long
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventID"
         m_strErrDesc = "The passed index number is not valid."
         EventID = 0
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventID = m_atypEventRecords(xi_lngIndex).EventID

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventData
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:34:44
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : LONG:
' Description  : READ-ONLY: Return the Event DATA of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventData(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventData"
         m_strErrDesc = "The passed index number is not valid."
         EventData = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventData = m_atypEventRecords(xi_lngIndex).EventData

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventDataText
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:34:44
' Inputs       : ByVal xi_lngIndex:Long -
' Outputs      : LONG:
' Description  : READ-ONLY: Return the Event DATA (text, not HEX) of an event log
'              :     record (based on passed index value)
'              : Check for zero values or values greater than
'              :     the number of log entries in the filtered
'              :     result set.
' *******************************************************
Public Property Get EventDataText(ByVal xi_lngIndex As Long) As String
   
   Select Case xi_lngIndex
      Case 0, Is > m_lngCount
         m_lngErrNumber = (ERR_BAD_INDEX + vbObjectError)
         m_strErrSource = EVENT_SOURCENAME & "EventDataText"
         m_strErrDesc = "The passed index number is not valid."
         EventDataText = vbNullString
         Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End Select

   EventDataText = m_atypEventRecords(xi_lngIndex).EventDataText

End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get EventTypeLog
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:20:42
' Inputs       : N/A
' Outputs      :
' Description  : Get/Let the Event Log Type property
' *******************************************************
Public Property Get EventTypeLog() As String
   EventTypeLog = m_strTypeEventLog
End Property

Public Property Let EventTypeLog(ByVal xi_strEventLogType As String)
   m_strTypeEventLog = xi_strEventLogType
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Get CountEventRecords
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:18:32
' Inputs       : N/A
' Outputs      : N/A
' Description  : READ-ONLY -- Returns the number of
'              :     event records
' *******************************************************
Public Property Get CountEventRecords() As Long
   CountEventRecords = m_lngCount
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Let EventReadLogForward
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:17:43
' Inputs       : ByVal xi_blnDirToRead:Boolean -
' Outputs      : N/A
' Description  : Set the direction to read the log -- forwards or
'              :     backwards.  Defaults to backwards, like the
'              :     NT Event Viewer
' *******************************************************
Public Property Let EventReadLogForward(ByVal xi_blnDirToRead As Boolean)
Attribute EventReadLogForward.VB_Description = "If you dont set the EventReadLogForward property, the server will read the event log items backwards (latest date first), which is the same as the default for the built-in NT Event Viewer program"
Attribute EventReadLogForward.VB_HelpID = 11
   m_blnEventReadLogForward = xi_blnDirToRead
End Property

Public Property Get EventReadLogForward() As Boolean
   EventReadLogForward = m_blnEventReadLogForward
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Let EventDataReturnHex
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:09:14
' Inputs       : ByVal xi_blnDataReturnHex:Boolean -
' Outputs      : N/A
' Description  : Set whether or not to return HEX values for the
'              :     data -- will only effect data over 256 bytes in
'              :     length (ie, if set to FALSE, the default, then
'              :     data 256 bytes or less will *still* be returned
'              :     as hex values separated by spaces)
' *******************************************************
Public Property Let EventDataReturnHex(ByVal xi_blnDataReturnHex As Boolean)
Attribute EventDataReturnHex.VB_Description = "False = return hex for Data items less than 256 bytes and untranslated strings for items over 256 bytes"
Attribute EventDataReturnHex.VB_HelpID = 23
   m_blnEventDataReturnHex = xi_blnDataReturnHex
End Property

Public Property Get EventDataReturnHex() As Boolean
   EventDataReturnHex = m_blnEventDataReturnHex
End Property

' ***********************************************************************
' ***********************************************************************
' ****                           CLASS Properties                    ****
' ***********************************************************************
' ***********************************************************************

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Sub Class_Initialize
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 12:11:17
' Inputs       : N/A
' Outputs      : N/A
' Description  : Indicates that an event log has never been
'              :     opened -- used to set correct error info
' *******************************************************
Private Sub Class_Initialize()
   m_lngEventLogHwnd = &HFFFF
End Sub
