Attribute VB_Name = "modFilterEvents"
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : FilterEvents.bas
' By        : L.J. Johnson       Date: 06-03-2001
' Comments  :
' ---------------------------------------------
Option Explicit

Public Const ALL_ITEMS                 As String = "[All]"

' *******************************************************
' Routine Name : (PUBLIC in FORM) FUNCTION FilterRecord
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/03/2001 -- 11:40:48
' Inputs       : ByRef xi_typRecord:EventRecord - Individual record (UDT)
' Outputs      : Boolean: TRUE if fits filter, FALSE otherwise
' Description  : Determine if this record fits the filter
' *******************************************************
Public Function FilterRecord(ByRef xi_typRecord As EventRecord, _
                             ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_lngLoop                       As Long
   Dim p_lngInnerLoop                  As Long
   Dim p_lngEventID                    As Long
   Dim p_lngCount                      As Long
   Dim p_blnShowRec                    As Boolean
   Dim p_blnInclude                    As Boolean
   Dim p_strEventType                  As String
   Dim p_strThisEventSource            As String
   Dim p_strThisEventCategory          As String
   Dim p_dtRecordDate                  As Date
   
   ' ------------------------------------------
   ' Assume we are going to show this record
   ' ------------------------------------------
   p_blnShowRec = True
   
   ' ------------------------------------------
   ' Check for Event Types
   ' ------------------------------------------
   p_strEventType = xi_typRecord.EventType
   p_blnShowRec = CheckEventTypes(xi_strEventType:=p_strEventType, _
                                  xi_typUserFilterData:=xi_typUserFilterData)
   
   ' ------------------------------------------
   ' Check for EventID
   ' ------------------------------------------
   If p_blnShowRec <> False Then
      p_lngEventID = xi_typRecord.EventID
      p_blnShowRec = CheckEventID(xi_lngEventID:=p_lngEventID, _
                                  xi_typUserFilterData:=xi_typUserFilterData)
   End If
   
   ' ------------------------------------------
   ' Check for Event Dates
   ' ------------------------------------------
   If p_blnShowRec <> False Then
      p_dtRecordDate = CDate(Format$(xi_typRecord.EventTimeCreated, "Short Date"))
      
      ' ---------------------------------------
      ' Only check if the 'Filter By Date'
      '     checkbox is checked!
      ' ---------------------------------------
      If xi_typUserFilterData.CategoryInclude = True Then
         p_blnShowRec = CheckEventDate(xi_dtRecordDate:=p_dtRecordDate, _
                                       xi_typUserFilterData:=xi_typUserFilterData)
      End If
   End If
   
   ' ------------------------------------------
   ' Check for Event Source
   ' ------------------------------------------
   If p_blnShowRec <> False Then
      p_strThisEventSource = xi_typRecord.EventSourceName
      p_blnInclude = xi_typUserFilterData.SourceInclude
      
      ' ---------------------------------------
      ' Get the name, and whether to include
      '     or exclude
      ' ---------------------------------------
      p_blnShowRec = GetSourcesMatch(xi_strThisEventSource:=p_strThisEventSource, _
                                     xi_blnInclude:=p_blnInclude, _
                                     xi_typUserFilterData:=xi_typUserFilterData)
   End If
   
   ' ------------------------------------------
   ' Check for Event Category
   ' ------------------------------------------
   If p_blnShowRec <> False Then
      p_strThisEventCategory = xi_typRecord.EventCategoryString
      p_blnInclude = xi_typUserFilterData.CategoryInclude
      
      ' ---------------------------------------
      ' Get the name, and whether to include
      '     or exclude
      ' ---------------------------------------
      p_blnShowRec = GetCategoriesMatch(xi_strThisEventCategory:=p_strThisEventCategory, _
                                        xi_blnInclude:=p_blnInclude, _
                                        xi_typUserFilterData:=xi_typUserFilterData)
   
   End If
   
   ' ------------------------------------------
   ' Set the return value
   ' ------------------------------------------
   FilterRecord = p_blnShowRec
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in FORM) Function CheckEventDate
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/03/2001 -- 12:05:30
' Inputs       : ByVal xi_dtRecordDate:Date - Date from the current record
' Outputs      : Boolen: TRUE if date falls within range, FALSE otherwise
' Description  : If filtering on dates, check if the
'              :     passed date falls within the range
'              :     from the frmFilterEvents form
' *******************************************************
Private Function CheckEventDate(ByVal xi_dtRecordDate As Date, _
                                ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_blnShowRec                    As Boolean
   
   ' ------------------------------------------
   ' Default to showing record
   ' ------------------------------------------
   p_blnShowRec = True
   
   If (xi_dtRecordDate < xi_typUserFilterData.EventDateFrom) Then
      p_blnShowRec = False
   ElseIf (xi_dtRecordDate > xi_typUserFilterData.EventDateTo) Then
      p_blnShowRec = False
   Else
      p_blnShowRec = True
   End If
   
   CheckEventDate = p_blnShowRec
            
End Function

' *******************************************************
' Routine Name : (PRIVATE in FORM) Function CheckEventID
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/03/2001 -- 11:59:14
' Inputs       : ByVal xi_lngEventID:Long - The EventID from current record
' Outputs      : Boolean: TRUE if event IDs match, FALSE otherwise
' Description  : If filtering on EventIDs, check to see
'              :     if the EventID of this record matches
'              :     the filter EventID
' *******************************************************
Private Function CheckEventID(ByVal xi_lngEventID As Long, _
                              ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_blnShowRec                    As Boolean
   
   ' ------------------------------------------
   ' Default to showing record
   ' ------------------------------------------
   p_blnShowRec = True
   
   If xi_typUserFilterData.EventID <> -1 Then
      
      If xi_typUserFilterData.EventIdInclude = True Then
         
         If xi_lngEventID <> xi_typUserFilterData.EventID Then
            p_blnShowRec = False
         End If
      
      ElseIf xi_typUserFilterData.EventIdInclude = False Then
         
         If xi_lngEventID = xi_typUserFilterData.EventID Then
            p_blnShowRec = False
         End If
      
      End If
   
   End If
   
   CheckEventID = p_blnShowRec
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in FORM) Function CheckEventTypes
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/03/2001 -- 11:49:27
' Inputs       : ByVal xi_strEventType:String - The EventType (string) of current record
' Outputs      : Boolean: TRUE if checked, FALSE otherwise
' Description  : Determine if the event type string passed
'              :     was checked or unchecked in the filter
'              :     screen
' *******************************************************
Private Function CheckEventTypes(ByVal xi_strEventType As String, _
                                 ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_blnShowRec                    As Boolean
   
   ' ------------------------------------------
   ' Default to showing record
   ' ------------------------------------------
   p_blnShowRec = True
   
   Select Case xi_strEventType
      Case Event_Type_Info
         If xi_typUserFilterData.EvtInfo = False Then
            p_blnShowRec = False
         End If
      
      Case Event_Type_Warning
         If xi_typUserFilterData.EvtWarning = False Then
            p_blnShowRec = False
         End If
      
      Case Event_Type_Error
         If xi_typUserFilterData.EvtError = False Then
            p_blnShowRec = False
         End If
      
      Case Event_Type_Success_Audit
         If xi_typUserFilterData.EvtSuccessAudit = False Then
            p_blnShowRec = False
         End If
      
      Case Event_Type_Failure_Audit
         If xi_typUserFilterData.EvtFailureAudit = False Then
            p_blnShowRec = False
         End If
      
      Case Else
         p_blnShowRec = False
   
   End Select
   
   CheckEventTypes = p_blnShowRec
   
End Function

' *******************************************************
' Routine Name : (PRIVATE in MODULE) Function GetCategoriesMatch
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/05/2001 -- 20:24:29
' Inputs       : ByVal xi_strThisEventCategory:String  - The current category string
'              : ByVal xi_blnInclude:Boolean           - Whether to INCLUDE or EXCLUDE
'              : ByRef xi_typUserFilterData:FilterData - The user's filter choices
' Outputs      : Boolean: If TRUE, then INCLUDE this event
' Description  : Determine if we should include or exclude
'              :     the event based on it's category string
' *******************************************************
Private Function GetCategoriesMatch(ByVal xi_strThisEventCategory As String, _
                                    ByVal xi_blnInclude As Boolean, _
                                    ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_strAllSources                 As String
   Dim p_strCategory1                  As String
   Dim p_strCategory2                  As String
   Dim p_strCategory3                  As String
   Dim p_strThisCategory               As String
   Dim p_lngPos                        As Long
   
   p_lngPos = InStr(1, xi_strThisEventCategory, ":")
   If p_lngPos > 0 Then
      p_strThisCategory = Trim$(Mid$(xi_strThisEventCategory, p_lngPos + 1))
   Else
      p_strThisCategory = Trim$(xi_strThisEventCategory)
   End If
   
   If (xi_typUserFilterData.Category1 = ALL_ITEMS) And _
      (xi_typUserFilterData.Category2 = ALL_ITEMS) And _
      (xi_typUserFilterData.Category3 = ALL_ITEMS) Then
      ' ---------------------------------------
      ' No filtering to do
      ' ---------------------------------------
      GetCategoriesMatch = True
      Exit Function
   End If
   
   If xi_typUserFilterData.Category1 <> ALL_ITEMS Then
      p_strAllSources = xi_typUserFilterData.Category1
      p_strCategory1 = xi_typUserFilterData.Category1
   Else
      p_strCategory1 = vbNullString
   End If
   
   If xi_typUserFilterData.Category2 <> ALL_ITEMS Then
      p_strAllSources = p_strAllSources & " " & xi_typUserFilterData.Category2
      p_strCategory2 = xi_typUserFilterData.Category2
   Else
      p_strCategory2 = vbNullString
   End If
   
   If xi_typUserFilterData.Category3 <> ALL_ITEMS Then
      p_strAllSources = p_strAllSources & " " & xi_typUserFilterData.Category3
      p_strCategory3 = xi_typUserFilterData.Category3
   Else
      p_strCategory3 = vbNullString
   End If
   
   ' ------------------------------------------
   ' Include one or more sources
   ' ------------------------------------------
   If xi_blnInclude = True Then
      If InStr(1, p_strAllSources, p_strThisCategory) <= 0 Then
         GetCategoriesMatch = False
      
      ' ---------------------------------------
      ' Handle things like 'DNS' vs 'DNS Server'
      ' ---------------------------------------
      Else
         If (p_strThisCategory = p_strCategory1) Or _
            (p_strThisCategory = p_strCategory2) Or _
            (p_strThisCategory = p_strCategory3) Then
            GetCategoriesMatch = True
         Else
            GetCategoriesMatch = False
         End If
      End If
            
   ' ------------------------------------------
   ' Exclude one or more sources
   ' ------------------------------------------
   ElseIf xi_blnInclude = False Then
      If InStr(1, p_strAllSources, p_strThisCategory) > 0 Then
      
         If (p_strThisCategory = p_strCategory1) Or _
            (p_strThisCategory = p_strCategory2) Or _
            (p_strThisCategory = p_strCategory3) Then
            GetCategoriesMatch = False
         Else
            GetCategoriesMatch = True
         End If
         
      ' ---------------------------------------
      ' Handle things like 'DNS' vs 'DNS Server'
      ' ---------------------------------------
      Else
         GetCategoriesMatch = True
         
      End If
      
   End If

End Function

' *******************************************************
' Routine Name : (PRIVATE in FORM) Function GetSourcesMatch
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/05/2001 -- 20:26:30
' Inputs       : ByVal xi_strThisEventSource:String    - The current category string
'              : ByVal xi_blnInclude:Boolean           - Whether in INCLUDE or EXCLUDE
'              : ByRef xi_typUserFilterData:FilterData - The user's filter choices
' Outputs      : Boolean: If TRUE, then INCLUDE this event
' Description  : Determine if we should include or exclude
'              :     the event based on it's source string
' *******************************************************
Private Function GetSourcesMatch(ByVal xi_strThisEventSource As String, _
                                 ByVal xi_blnInclude As Boolean, _
                                 ByRef xi_typUserFilterData As FilterData) As Boolean
   Dim p_strAllSources                 As String
   Dim p_strSource1                    As String
   Dim p_strSource2                    As String
   Dim p_strSource3                    As String
   
   If (xi_typUserFilterData.Source1 = ALL_ITEMS) And _
      (xi_typUserFilterData.Source2 = ALL_ITEMS) And _
      (xi_typUserFilterData.Source3 = ALL_ITEMS) Then
      ' ---------------------------------------
      ' No filtering to do
      ' ---------------------------------------
      GetSourcesMatch = True
      Exit Function
   End If
   
   If xi_typUserFilterData.Source1 <> ALL_ITEMS Then
      p_strAllSources = "||" & xi_typUserFilterData.Source1
      p_strSource1 = xi_typUserFilterData.Source1
   Else
      p_strSource1 = vbNullString
   End If
   
   If xi_typUserFilterData.Source2 <> ALL_ITEMS Then
      p_strAllSources = p_strAllSources & "||" & xi_typUserFilterData.Source2
      p_strSource2 = xi_typUserFilterData.Source2
   Else
      p_strSource2 = vbNullString
   End If
   
   If xi_typUserFilterData.Source3 <> ALL_ITEMS Then
      p_strAllSources = p_strAllSources & "||" & xi_typUserFilterData.Source3
      p_strSource3 = xi_typUserFilterData.Source3
   Else
      p_strSource3 = vbNullString
   End If
   
   ' ------------------------------------------
   ' Include one or more sources
   ' ------------------------------------------
   If xi_blnInclude = True Then
      If InStr(1, p_strAllSources, "||" & xi_strThisEventSource) <= 0 Then
         GetSourcesMatch = False
      
      ' ---------------------------------------
      ' Handle things like 'DNS' vs 'DNS Server'
      ' ---------------------------------------
      Else
         If (xi_strThisEventSource = p_strSource1) Or _
            (xi_strThisEventSource = p_strSource2) Or _
            (xi_strThisEventSource = p_strSource3) Then
            GetSourcesMatch = True
         Else
            GetSourcesMatch = False
         End If
      End If
            
   ' ------------------------------------------
   ' Exclude one or more sources
   ' ------------------------------------------
   ElseIf xi_blnInclude = False Then
      If InStr(1, p_strAllSources, "||" & xi_strThisEventSource) > 0 Then
      
         If (xi_strThisEventSource = p_strSource1) Or _
            (xi_strThisEventSource = p_strSource2) Or _
            (xi_strThisEventSource = p_strSource3) Then
            GetSourcesMatch = False
         Else
            GetSourcesMatch = True
         End If
         'GetSourcesMatch = False
         
      ' ---------------------------------------
      ' Handle things like 'DNS' vs 'DNS Server'
      ' ---------------------------------------
      Else
         GetSourcesMatch = True
         
      End If
      
   End If

End Function
