Attribute VB_Name = "modMain"
' ************************************************************
' Copyright  1995-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : modMain.BAS
' By        : L.J. Johnson       Date: 05-28-2001
' Comments  :
' ---------------------------------------------
Option Explicit

' ---------------------------------------------
' Used to get error messages directly from the
'    system instead of hard-coding them
' ---------------------------------------------
Private Const FORMAT_MESSAGE_FROM_SYSTEM     As Long = &H1000&
Private Const FORMAT_MESSAGE_IGNORE_INSERTS  As Long = &H200&
Private Const FORMAT_MESSAGE_FROM_HMODULE    As Long = &H800&
Private Const LOAD_LIBRARY_AS_DATAFILE       As Long = 2&
Private Const MAX_PATH                       As Long = 260&

' ---------------------------------------------
' Custom error messages for this app
' ---------------------------------------------
Public Const ERR_REG_EVENT_SOURCE      As Long = 9001&
Public Const ERR_REPORT_EVENT          As Long = 9002&
Public Const ERR_DEREG_EVENT_SOURCE    As Long = 9003&
Public Const ERR_NO_CREATE_KEY         As Long = 9004&
Public Const ERR_NO_OPEN_KEY           As Long = 9005&
Public Const ERR_NO_SET_FIRST_VALUE    As Long = 9006&
Public Const ERR_NO_SET_SECOND_VALUE   As Long = 9007&
Public Const ERR_NO_CLOSE_KEY          As Long = 9008&

' ---------------------------------------------
' Status Codes
' ---------------------------------------------
Private Const INVALID_HANDLE_VALUE           As Long = -1&
Public Const ERROR_SUCCESS                   As Long = 0&

' ---------------------------------------------
' Upper and lower bounds of network errors
' ---------------------------------------------
Private Const NERR_BASE                      As Long = 2100&
Private Const MAX_NERR                       As Long = NERR_BASE + 899&

' ---------------------------------------------
' Upper and lower bounds of Internet errors
' ---------------------------------------------
Private Const INTERNET_ERROR_BASE            As Long = 12000&
Private Const INTERNET_ERROR_LAST            As Long = INTERNET_ERROR_BASE + 171&

Private Declare Function GetShortPathName _
   Lib "Kernel32" Alias "GetShortPathNameA" _
   (ByVal lpszLongPath As String, _
    ByVal lpszShortPath As String, _
    ByVal nBufferLength As Long) As Long
Private Declare Function FormatMessage _
   Lib "Kernel32" Alias "FormatMessageA" _
   (ByVal dwFlags As Long, _
    lpSource As Any, _
    ByVal dwMessageId As Long, _
    ByVal dwLanguageId As Long, _
    ByVal lpBuffer As String, _
    ByVal nSize As Long, _
    Arguments As Long) As Long
Private Declare Function LoadLibraryEx _
   Lib "Kernel32" Alias "LoadLibraryExA" _
   (ByVal lpLibFileName As String, _
    ByVal hFile As Long, _
    ByVal dwFlags As Long) As Long
Private Declare Function FreeLibrary _
   Lib "Kernel32" _
   (ByVal hLibModule As Long) As Long
Private Declare Function MultiByteToWideChar _
   Lib "Kernel32" _
   (ByVal CodePage As Long, _
    ByVal dwFlags As Long, _
    lpMultiByteStr As Any, _
    ByVal cchMultiByte As Long, _
    lpWideCharStr As Any, _
    ByVal cchWideChar As Long) As Long
Private Declare Function lstrcpyA _
   Lib "Kernel32" _
   (lpString1 As Any, _
    lpString2 As Any) As Long
Private Declare Function lstrlenA _
   Lib "Kernel32" _
   (lpString As Any) As Long

Private Const CP_ACP                   As Long = 0&   ' ANSI code page

' -------------------------------------------------
' The types of events that can be logged.
' -------------------------------------------------
Public Const EVENTLOG_SUCCESS = &H0
Public Const EVENTLOG_ERROR_TYPE = &H1
Public Const EVENTLOG_WARNING_TYPE = &H2
Public Const EVENTLOG_INFORMATION_TYPE = &H4
Public Const EVENTLOG_AUDIT_SUCCESS = &H8
Public Const EVENTLOG_AUDIT_FAILURE = &H10

' -------------------------------------------------
' Constants for different types of events
' -------------------------------------------------
Public Const Event_Type_Info = "Information"
Public Const Event_Type_Warning = "Warning"
Public Const Event_Type_Error = "Error"
Public Const Event_Type_Success_Audit = "Success Audit"
Public Const Event_Type_Failure_Audit = "Failure Audit"

' -------------------------------------------------
' Constants for the type of log you want to look
'   at.  Note that user program will need to use
'   these constants also
' -------------------------------------------------
Public Const EVNT_SYSTEM = "System"
Public Const EVNT_APP = "Application"
Public Const EVNT_SECURITY = "Security"

' -------------------------------------------------
' Custom error return codes for the various class
'   modules
' -------------------------------------------------
Public Const ERR_LOGTYPE_NOT_SET = 1011&
Public Const ERR_SOURCENAME_NOT_SET = 1012&
Public Const ERR_BAD_INDEX = 1013&
Public Const ERR_FAILED_OPEN_REGISTRY_KEY = 1014&
Public Const ERR_FAILED_READ_REGISTRY_KEY = 1015&
Public Const ERR_RESOURCE_DATA_NOT_FOUND = 1016&
Public Const ERR_READING_EVENT_LOG = 1017&
Public Const ERR_LOG_NOT_OPENED = 1018&
Public Const ERR_FAILED_SET_LOG_TYPE = 1019&
Public Const ERR_FAILED_FORMAT_MSG = 1020&
Public Const ERR_BAD_SERVER_NAME = 1021&
Public Const ERR_READ_EVENT_RECORD = 1022&

' -------------------------------------------------
' API calls that need to be public within program
' -------------------------------------------------
Public Declare Sub CopyMem _
   Lib "Kernel32" Alias "RtlCopyMem" _
   (pTo As Any, _
    uFrom As Any, _
    ByVal lSize As Long)
Public Declare Sub MoveMem _
   Lib "Kernel32" Alias "RtlMoveMemory" _
   (pTo As Any, _
    uFrom As Any, _
    ByVal lSize As Long)

' --------------------------------------------------------
' Eventlog Status Codes
' --------------------------------------------------------
Public Const ERROR_FILE_NOT_FOUND = 2&
Public Const ERROR_PATH_NOT_FOUND = 3&
Public Const ERROR_TOO_MANY_OPEN_FILES = 4&
Public Const ERROR_ACCESS_DENIED = 5&
Public Const ERROR_INVALID_HANDLE = 6&
Public Const ERROR_ARENA_TRASHED = 7&
Public Const ERROR_NOT_ENOUGH_MEMORY = 8&
Public Const ERROR_OUTOFMEMORY = 14&
Public Const ERROR_HANDLE_EOF = 38&
Public Const ERROR_INVALID_PARAMETER = 87&
Public Const ERROR_INSUFFICIENT_BUFFER = 122&
Public Const ERROR_INVALID_NAME = 123&
Public Const ERROR_ALREADY_EXISTS = 183&
Public Const ERROR_MORE_DATA = 234&
Public Const ERROR_NO_MORE_ITEMS = 259&
Public Const ERROR_MR_MID_NOT_FOUND = 317&
Public Const ERROR_NOACCESS = 998&
Public Const ERROR_BADDB = 1009&
Public Const ERROR_BADKEY = 1010&
Public Const ERROR_CANTOPEN = 1011&
Public Const ERROR_CANTREAD = 1012&
Public Const ERROR_CANTWRITE = 1013&
Public Const ERROR_EVENTLOG_FILE_CORRUPT = 1500&
Public Const ERROR_EVENTLOG_CANT_START = 1501&
Public Const ERROR_LOG_FILE_FULL = 1502&
Public Const ERROR_EVENTLOG_FILE_CHANGED = 1503&
Public Const RPC_S_SERVER_UNAVAILABLE = 1722&
Public Const ERROR_RESOURCE_DATA_NOT_FOUND = 1812&

' --------------------------------------------
'
' --------------------------------------------
Private Type FILETIME
   dwLowDateTime                       As Long
   dwHighDateTime                      As Long
End Type

Private Type SYSTEMTIME
  wYear                                As Integer
  wMonth                               As Integer
  wDayOfWeek                           As Integer
  wDay                                 As Integer
  wHour                                As Integer
  wMinute                              As Integer
  wSecond                              As Integer
  wMilliseconds                        As Integer
End Type

Private Declare Function SystemTimeToFileTime _
   Lib "Kernel32" _
   (lpSystemTime As SYSTEMTIME, _
    lpFileTime As FILETIME) As Long
Private Declare Function FileTimeToLocalFileTime _
   Lib "Kernel32" _
   (lpFileTime As FILETIME, _
    lpLocalFileTime As FILETIME) As Long
Private Declare Function FileTimeToSystemTime _
   Lib "Kernel32" _
   (lpFileTime As FILETIME, _
    lpSystemTime As SYSTEMTIME) As Long
Private Declare Function VariantTimeToSystemTime _
   Lib "oleaut32" _
   (ByVal vtime As Double, _
    psystime As SYSTEMTIME) As Long
Private Declare Function SystemTimeToVariantTime _
   Lib "oleaut32" _
   (psystime As SYSTEMTIME, _
    pvtime As Double) As Long

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function LoWord
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 01/16/1999 -- 12:59:23
' Inputs       : lngNumber:Long - A long (4 bytes)
' Outputs      : Long --
' Description  : Returns the low-word portion of a long
' *******************************************************
Public Function LoWord(ByVal lngNumber As Long) As Long
   LoWord = lngNumber And &HFFFF&
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function HiWord
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 01/16/1999 -- 12:59:21
' Inputs       : lngNumber:Long - A long (4 bytes)
' Outputs      : Long --
' Description  : Returns the high-word portion of a long
' *******************************************************
Public Function HiWord(lngNumber As Long) As Long
   HiWord = lngNumber / &H10000
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function DateToLocalDate
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 15:37:35
' Inputs       : xi_dteTimes:Date -
' Outputs      : N/A
' Description  : Converts a date/time from UCT (Universal
'              :     Coordiated Time) to the local time
'              :     of this machine
'              : Dates for the event log items are stored
'              :     in UCT format
'              : Thanks to Brad Martinez for this code
' *******************************************************
Public Function DateToLocalDate(ByVal xi_dteTimes As Date) As Date
   Dim p_typSystemTime                 As SYSTEMTIME
   Dim p_typFileTime                   As FILETIME
   Dim p_typLocalFileTime              As FILETIME
   Dim p_dblReturn                     As Double
  
   VariantTimeToSystemTime CDbl(xi_dteTimes), p_typSystemTime
   SystemTimeToFileTime p_typSystemTime, p_typFileTime
   FileTimeToLocalFileTime p_typFileTime, p_typLocalFileTime
   FileTimeToSystemTime p_typLocalFileTime, p_typSystemTime
   SystemTimeToVariantTime p_typSystemTime, p_dblReturn
   DateToLocalDate = p_dblReturn
  
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function GetStrFromPtrA
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 15:52:43
' Inputs       : ByVal xi_lngPtrString:Long -
'              : Optional ByVal xi_lngStrLen:Long = 0 -
' Outputs      : N/A
' Description  : Return a string given a pointer to a
'              :     string
'              : Thanks to Brad Martinez for this code --
'              :     had to add an optional parameter to
'              :     handle items with included nulls
' *******************************************************
Public Function GetStrFromPtrA(ByVal xi_lngPtrString As Long, _
                               Optional ByVal xi_lngStrLen As Long = 0) As String
   Dim p_lngNumChars                   As Long
  
   If xi_lngStrLen = 0 Then
      p_lngNumChars = lstrlenA(ByVal xi_lngPtrString)
   Else
      p_lngNumChars = xi_lngStrLen
   End If
   
   GetStrFromPtrA = Space$(p_lngNumChars)
   MultiByteToWideChar CP_ACP, _
                       0, _
                       ByVal xi_lngPtrString, _
                       p_lngNumChars, _
                       ByVal StrPtr(GetStrFromPtrA), _
                       p_lngNumChars

End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function ClearLastCrLf
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 15:48:54
' Inputs       : xi_strTmpMsg:String -
' Outputs      : STRING:
' Description  : Clear any trailing CRs, LFs or CRLFs
'              :     from the passed string
' *******************************************************
Public Function ClearLastCrLf(ByVal xi_strTmpMsg As String) As String
   Dim p_strTmpMsg                     As String
   
   ' ----------------------------------------------
   ' Make a temporary copy of the passed string
   ' ----------------------------------------------
   p_strTmpMsg = Trim$(xi_strTmpMsg)
   
   ' ----------------------------------------------
   ' Trim any trailing nulls
   ' ----------------------------------------------
   If Right$(p_strTmpMsg, 1) = Chr$(0) Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   End If
   
   ' ----------------------------------------------
   ' Trim any trailing Cr's, Lf's, or CrLf's
   ' ----------------------------------------------
   If Right$(p_strTmpMsg, 2) = vbCrLf Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 2)
   ElseIf Right$(p_strTmpMsg, 1) = vbLf Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   ElseIf Right$(p_strTmpMsg, 1) = vbCr Then
      p_strTmpMsg = Mid$(p_strTmpMsg, 1, Len(p_strTmpMsg) - 1)
   End If

   ' ----------------------------------------------
   ' Set the return value
   ' ----------------------------------------------
   ClearLastCrLf = p_strTmpMsg
   
End Function

Public Function GetShortName(ByVal xi_strLongPathName As String) As String
   Dim p_strBuffer                     As String
   Dim p_lngRtn                        As Long
   
   p_strBuffer = Space$(MAX_PATH)
   If Right$(xi_strLongPathName, 1) = "\" Then
      xi_strLongPathName = Mid$(xi_strLongPathName, 1, Len(xi_strLongPathName) - 1)
   End If
   
   p_lngRtn = GetShortPathName(lpszLongPath:=xi_strLongPathName, _
                               lpszShortPath:=p_strBuffer, _
                               nBufferLength:=Len(p_strBuffer))
   If p_lngRtn > 0 Then
      GetShortName = Trim$(Mid$(p_strBuffer, 1, p_lngRtn))
   Else
      GetShortName = ""
   End If
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function ReturnApiErrString
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 01/16/1999 -- 12:56:46
' Inputs       : ErrorCode:Long - Number returned from API error
' Outputs      : N/A
' Description  : Function returns the error string
'              : The original code appeared in Keith Pleas
'              :     article in VBPJ, April 1996 (OLE Expert
'              :     column).  Thanks, Keith.
' *******************************************************
Public Function ReturnApiErrString(ErrorCode As Long) As String
On Error Resume Next                   ' Don't accept an error here
   Dim p_strBuffer                     As String
   Dim p_lngHwndModule                 As Long
   Dim p_lngFlags                      As Long
   
   ' ------------------------------------------
   ' Separate handling for network errors
   ' netmsg.dll
   ' ------------------------------------------
   If ErrorCode >= NERR_BASE And _
      ErrorCode <= MAX_NERR Then
      
      p_lngHwndModule = LoadLibraryEx(lpLibFileName:="netmsg.dll", _
                        hFile:=0&, _
                        dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
      
      If p_lngHwndModule <> 0 Then
      
         p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                      FORMAT_MESSAGE_IGNORE_INSERTS Or _
                      FORMAT_MESSAGE_FROM_HMODULE
                      
         ' ------------------------------------
         ' Allocate the string, then get the
         '     system to tell us the error
         '     message associated with this error number
         ' ------------------------------------
         p_strBuffer = String(256, 0)
         FormatMessage dwFlags:=p_lngFlags, _
                       lpSource:=ByVal p_lngHwndModule, _
                       dwMessageId:=ErrorCode, _
                       dwLanguageId:=0&, _
                       lpBuffer:=p_strBuffer, _
                       nSize:=Len(p_strBuffer), _
                       Arguments:=ByVal 0&
      
         ' ------------------------------------
         ' Strip the last null, then the last
         '     CrLf pair if it exists
         ' ------------------------------------
         p_strBuffer = Left(p_strBuffer, InStr(p_strBuffer, vbNullChar) - 1)
         If Right$(p_strBuffer, 2) = Chr$(13) & Chr$(10) Then
            p_strBuffer = Mid$(p_strBuffer, 1, Len(p_strBuffer) - 2)
         End If
         
         FreeLibrary hLibModule:=p_lngHwndModule
      End If
   
   ' ------------------------------------------
   ' Separate handling for Wininet error
   ' Wininet.dll
   ' ------------------------------------------
   ElseIf ErrorCode >= INTERNET_ERROR_BASE And _
      ErrorCode <= INTERNET_ERROR_LAST Then
      
      ' ---------------------------------------
      ' Load the library
      ' ---------------------------------------
      p_lngHwndModule = LoadLibraryEx(lpLibFileName:="Wininet.dll", _
                        hFile:=0&, _
                        dwFlags:=LOAD_LIBRARY_AS_DATAFILE)
      
      If p_lngHwndModule <> 0 Then
      
         p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                      FORMAT_MESSAGE_IGNORE_INSERTS Or _
                      FORMAT_MESSAGE_FROM_HMODULE
                      
         ' ------------------------------------
         ' Allocate the string, then get the
         '     system to tell us the error
         '     message associated with this error number
         ' ------------------------------------
         p_strBuffer = String(256, 0)
         FormatMessage dwFlags:=p_lngFlags, _
                       lpSource:=ByVal p_lngHwndModule, _
                       dwMessageId:=ErrorCode, _
                       dwLanguageId:=0&, _
                       lpBuffer:=p_strBuffer, _
                       nSize:=Len(p_strBuffer), _
                       Arguments:=ByVal 0&
      
         ' ------------------------------------
         ' Strip the last null, then the last
         '     CrLf pair if it exists
         ' ------------------------------------
         p_strBuffer = Left(p_strBuffer, InStr(p_strBuffer, vbNullChar) - 1)
         If Right$(p_strBuffer, 2) = Chr$(13) & Chr$(10) Then
            p_strBuffer = Mid$(p_strBuffer, 1, Len(p_strBuffer) - 2)
         End If
         
         FreeLibrary hLibModule:=p_lngHwndModule
      End If
   
   ' ------------------------------------------
   ' Wasn't Wininet or NetMsg, so do the standard
   '     API error look-up
   ' ------------------------------------------
   Else
      ' ---------------------------------------
      ' Allocate the string, then get the system
      '     to tell us the error message associated
      '     with this error number
      ' ---------------------------------------
      p_strBuffer = String(256, 0)
      p_lngFlags = FORMAT_MESSAGE_FROM_SYSTEM Or _
                   FORMAT_MESSAGE_IGNORE_INSERTS
      
      FormatMessage dwFlags:=p_lngFlags, _
                    lpSource:=ByVal 0&, _
                    dwMessageId:=ErrorCode, _
                    dwLanguageId:=0&, _
                    lpBuffer:=p_strBuffer, _
                    nSize:=Len(p_strBuffer), _
                    Arguments:=ByVal 0&
   
      ' ---------------------------------------
      ' Strip the last null, then the last CrLf
      '     pair if it exists
      ' ------------------------------------------
      p_strBuffer = Left(p_strBuffer, InStr(p_strBuffer, vbNullChar) - 1)
      If Right$(p_strBuffer, 2) = Chr$(13) & Chr$(10) Then
         p_strBuffer = Mid$(p_strBuffer, 1, Len(p_strBuffer) - 2)
      End If
   End If
   
   ' ------------------------------------------
   ' Set the return value
   ' ------------------------------------------
   ReturnApiErrString = p_strBuffer

End Function
