VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cRegistryDB"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = True
' ************************************************************
' Copyright  1999-2001 Slightly Tilted Software
' All rights reserved
' You're absolutely free to use these resources within your
'     own applications, but you may not redistribute them
'     (as source) in any manner whatsoever, whether for profit
'     or not.
' The only legitimate source for the original source code is
'     at the VBPJ site and my own web site at:
'     http://www.SlightlyTiltedSoftware.com
' ************************************************************

' ---------------------------------------------
' Module    : Register_DB.cls
' By        : L.J. Johnson       Date: 04-28-2001
' Comments  : Retrieve registry entries for the
'           :     EventMessageFile or PamameterMessageLog
'           : Get the Event Log Names for this
'           :     system
' ---------------------------------------------
Option Explicit

' -------------------------------------------------
' For keeping track of our own error info
' -------------------------------------------------
Private m_lngErrNumber                 As Long
Private m_strErrSource                 As String
Private m_strErrDesc                   As String

' --------------------------------------------
'
' --------------------------------------------
Private Const ERROR_SUCCESS            As Long = 0&
Private Const ERR_NO_OPEN_KEY          As Long = 9005&
Private Const ERR_NO_CLOSE_KEY         As Long = 9008&

' --------------------------------------------
' Used as part of error string return
' --------------------------------------------
Private Const REG_SOURCENAME           As String = "ReadEventLogs.RegistryDB."

' --------------------------------------------
' Type of item to look up
' --------------------------------------------
Public Enum enmMessageLookup
   EventMessage = 0&
   ParameterMessage = 1&
   CategoryMessage = 2&
End Enum

' --------------------------------------------
' Module-level variables used to feed the Property
'   Get functions
' --------------------------------------------
Private m_strLogType                   As String
Private m_strSourceName                As String
Private m_strRegistryEntry             As String

' --------------------------------------------
'
' --------------------------------------------
Private Type FILETIME
   dwLowDateTime                       As Long
   dwHighDateTime                      As Long
End Type

' --------------------------------------------
' These 32-bit API calls are used only in this
'   class module
' --------------------------------------------
Private Declare Function RegConnectRegistry _
   Lib "advapi32.dll" Alias "RegConnectRegistryA" _
   (ByVal lpMachineName As String, _
    ByVal hKey As Long, _
    phkResult As Long) As Long
Private Declare Function RegCloseKey _
   Lib "advapi32" _
   (ByVal hKey As Long) As Long
Private Declare Function RegOpenKeyEx _
   Lib "advapi32" Alias "RegOpenKeyExA" _
   (ByVal hKey As Long, _
    ByVal lpSubKey As String, _
    ByVal ulOptions As Long, _
    ByVal samDesired As Long, _
    phkResult As Long) As Long
Private Declare Function RegQueryValueEx _
   Lib "advapi32" Alias "RegQueryValueExA" _
   (ByVal hKey As Long, _
    ByVal lpValueName As String, _
    lpReserved As Long, _
    lptype As Long, _
    lpData As Any, _
    lpcbData As Long) As Long
Private Declare Function RegEnumKeyEx _
   Lib "advapi32.dll" Alias "RegEnumKeyExA" _
   (ByVal hKey As Long, _
    ByVal dwIndex As Long, _
    ByVal lpName As String, _
    lpcbName As Long, _
    ByVal lpReserved As Long, _
    ByVal lpClass As String, _
    lpcbClass As Long, _
    lpftLastWriteTime As FILETIME) As Long
Private Declare Function RegEnumKey _
   Lib "advapi32.dll" Alias "RegEnumKeyA" _
   (ByVal hKey As Long, _
    ByVal dwIndex As Long, _
    ByVal lpName As String, _
    ByRef cbName As Long) As Long

' --------------------------------------------
' These constants for main registry keys are used
'   only in this class module
' --------------------------------------------
Private Const HKEY_CLASSES_ROOT = &H80000000
Private Const HKEY_CURRENT_USER = &H80000001
Private Const HKEY_LOCAL_MACHINE = &H80000002
Private Const HKEY_USERS = &H80000003
Private Const HKEY_PERFORMANCE_DATA = &H80000004
Private Const HKEY_CURRENT_CONFIG = &H80000005
Private Const HKEY_DYN_DATA = &H80000006

' --------------------------------------------
' Read/Write permissions for registry -- used only
'   in this class module
' --------------------------------------------
Private Const KEY_QUERY_VALUE = &H1&
Private Const KEY_SET_VALUE = &H2&
Private Const KEY_CREATE_SUB_KEY = &H4&
Private Const KEY_ENUMERATE_SUB_KEYS = &H8&
Private Const KEY_NOTIFY = &H10&
Private Const KEY_CREATE_LINK = &H20&
Private Const READ_CONTROL = &H20000
Private Const WRITE_DAC = &H40000
Private Const WRITE_OWNER = &H80000
Private Const SYNCHRONIZE = &H100000
Private Const STANDARD_RIGHTS_REQUIRED = &HF0000
Private Const STANDARD_RIGHTS_READ = READ_CONTROL
Private Const STANDARD_RIGHTS_WRITE = READ_CONTROL
Private Const STANDARD_RIGHTS_EXECUTE = READ_CONTROL
Private Const KEY_READ = STANDARD_RIGHTS_READ Or _
                         KEY_QUERY_VALUE Or _
                         KEY_ENUMERATE_SUB_KEYS Or _
                         KEY_NOTIFY
Private Const KEY_WRITE = STANDARD_RIGHTS_WRITE Or _
                          KEY_SET_VALUE Or _
                          KEY_CREATE_SUB_KEY
Private Const KEY_EXECUTE = KEY_READ

Private Const REG_EXPAND_SZ            As Long = 2&
Private Const REG_DWORD                As Long = 4&
Private Const REG_MULTI_SZ             As Long = 7&

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function GetSourceNames
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 06/02/2001 -- 14:13:11
' Inputs       : ByVal xi_strLogName:String -
'              : Optional ByVal xi_strServerName:String = "" -
' Outputs      : N/A
' Description  :
'              :
'              :
' *******************************************************
Public Function GetSourceNames(ByVal xi_strLogName As String, _
                               Optional ByVal xi_strServerName As String = "") As Variant
   Dim p_strSubKey                     As String
   Dim p_strMulti                      As String
   Dim p_avntSourceNames()             As String
   Dim p_lngRtn                        As Long
   Dim p_lngKeyHandle                  As Long
   Dim p_lngLenMulti                   As Long
   Dim p_lngRemoteHandle               As Long
   Const BASE_DIR                      As String = "System\CurrentControlSet\Services\EventLog\"

   ' ------------------------------------------
   '
   ' ------------------------------------------
   If Len(Trim$(xi_strServerName)) > 0 Then
      If Left$(xi_strServerName, 2) <> "\\" Then
         xi_strServerName = xi_strServerName & "\\"
      End If
   
      p_lngRtn = RegConnectRegistry(lpMachineName:=xi_strServerName, _
                                   hKey:=HKEY_LOCAL_MACHINE, _
                                   phkResult:=p_lngRemoteHandle)
   Else
      p_lngRemoteHandle = 0&
   End If
   
   ' ------------------------------------------
   ' Set the full subkey name
   ' ------------------------------------------
   p_strSubKey = BASE_DIR & xi_strLogName
   
   ' ------------------------------------------
   ' Open the key so we can enumerate it's keys
   ' ------------------------------------------
   If p_lngRemoteHandle = 0& Then
      p_lngRtn = RegOpenKeyEx(hKey:=HKEY_LOCAL_MACHINE, _
                              lpSubKey:=p_strSubKey, _
                              ulOptions:=0&, _
                              samDesired:=KEY_READ, _
                              phkResult:=p_lngKeyHandle)
   Else
      p_lngRtn = RegOpenKeyEx(hKey:=p_lngRemoteHandle, _
                              lpSubKey:=p_strSubKey, _
                              ulOptions:=0&, _
                              samDesired:=KEY_READ, _
                              phkResult:=p_lngKeyHandle)
   End If
   If p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_NO_OPEN_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "GetSourceNames"
      m_strErrDesc = "Could not OPEN the key, " & p_strSubKey & _
                     ", in the registry" & ".  The error was: " & _
                     ReturnApiErrString(p_lngRtn)
      Exit Function
   End If

   ' ------------------------------------------
   ' Get the current values for 'Sources'
   ' ------------------------------------------
   p_strMulti = Space$(2048 * 10)
   p_lngLenMulti = Len(p_strMulti)
   p_lngRtn = RegQueryValueEx(hKey:=p_lngKeyHandle, _
                              lpValueName:="Sources", _
                              lpReserved:=ByVal 0&, _
                              lptype:=REG_MULTI_SZ, _
                              lpData:=ByVal p_strMulti, _
                              lpcbData:=p_lngLenMulti)
   If p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_NO_OPEN_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "GetSourceNames"
      m_strErrDesc = "Could not READ the value, 'Sources', " & _
                     "in the registry at HKLM\" & p_strSubKey & "." & vbCrLf & "The error was: " & _
                     ReturnApiErrString(p_lngRtn)
      Exit Function
   Else
      p_strMulti = Left$(p_strMulti, p_lngLenMulti)
      p_avntSourceNames = Split(p_strMulti, Chr$(0))
   End If

   p_lngRtn = RegCloseKey(p_lngKeyHandle)
   
   GetSourceNames = p_avntSourceNames
   
End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Function GetLogNames
' Written By   : L.J. Johnson
' Programmer   : L.J. Johnson [Slightly Tilted Software]
' Date Writen  : 04/28/2001 -- 18:05:15
' Inputs       : N/A
' Outputs      : Variant --
' Description  : Get the names of the event logs available
'              :     on this system
'              : Applies to W2K+, defaults for NT 4.x
' Called By    : *****
' *******************************************************
Public Function GetLogNames(Optional ByVal xi_strServerName As String = "") As Variant
   Dim p_lngRtn                        As Long
   Dim p_lngLenKeyName                 As Long
   Dim p_lngIndex                      As Long
   Dim p_lngNumItems                   As Long
   Dim p_lngKeyHandle                  As Long
   Dim p_lngRemoteHandle               As Long
   Dim p_strKeyName                    As String
   Dim p_strSubKey                     As String
   Dim p_typLastWriteTime              As FILETIME
   Dim p_vntNames()                    As Variant
   
   Const BASE_DIR                      As String = "System\CurrentControlSet\Services\EventLog"
   Const NO_MORE_ITEMS                 As Long = 259&
   
   ' ------------------------------------------
   ' Set the full subkey name
   ' ------------------------------------------
   p_strSubKey = BASE_DIR

   ' ------------------------------------------
   '
   ' ------------------------------------------
   If Len(Trim$(xi_strServerName)) > 0 Then
      If Left$(xi_strServerName, 2) <> "\\" Then
         xi_strServerName = xi_strServerName & "\\"
      End If
   
      p_lngRtn = RegConnectRegistry(lpMachineName:=xi_strServerName, _
                                   hKey:=HKEY_LOCAL_MACHINE, _
                                   phkResult:=p_lngRemoteHandle)
   Else
      p_lngRemoteHandle = 0&
   End If
   
   ' ------------------------------------------
   ' Open the key so we can enumerate it's keys
   ' ------------------------------------------
   If p_lngRemoteHandle = 0& Then
      p_lngRtn = RegOpenKeyEx(hKey:=HKEY_LOCAL_MACHINE, _
                              lpSubKey:=p_strSubKey, _
                              ulOptions:=0&, _
                              samDesired:=KEY_READ, _
                              phkResult:=p_lngKeyHandle)
   Else
      p_lngRtn = RegOpenKeyEx(hKey:=p_lngRemoteHandle, _
                              lpSubKey:=p_strSubKey, _
                              ulOptions:=0&, _
                              samDesired:=KEY_READ, _
                              phkResult:=p_lngKeyHandle)
   End If
   If p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_NO_OPEN_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegisterEventSource"
      m_strErrDesc = "Could not OPEN the list of event log names in " & _
                         BASE_DIR & ".  The error was: " & _
                         ReturnApiErrString(p_lngRtn)
      Exit Function
   End If

   ' ------------------------------------------
   ' Now, enumerate them
   ' ------------------------------------------
   Do
      p_lngLenKeyName = 255
      p_strKeyName = String$(p_lngLenKeyName, 0)
      p_lngRtn = RegEnumKey(hKey:=p_lngKeyHandle, _
                            dwIndex:=p_lngIndex, _
                            lpName:=p_strKeyName, _
                            cbName:=p_lngLenKeyName)
      
      If p_lngRtn = NO_MORE_ITEMS Then
         GetLogNames = p_vntNames
         GoTo CloseKey
      
      ElseIf p_lngRtn = ERROR_SUCCESS Then
         ReDim Preserve p_vntNames(0 To p_lngNumItems)
         p_vntNames(p_lngNumItems) = Mid$(p_strKeyName, 1, p_lngLenKeyName)
         p_lngNumItems = p_lngNumItems + 1
         p_lngIndex = p_lngIndex + 1
         
      Else
         m_lngErrNumber = (Err.LastDllError + vbObjectError)
         m_strErrSource = REG_SOURCENAME & "GetLogName"
         m_strErrDesc = "Error retriving event log names, " & _
                            ReturnApiErrString(Err.LastDllError)
         Err.Raise Number:=m_lngErrNumber, _
                   Description:=m_strErrDesc, _
                   Source:=m_strErrSource
      End If

   Loop While p_lngRtn <> NO_MORE_ITEMS
   
CloseKey:
   If p_lngRemoteHandle <> 0 Then
      p_lngRtn = RegCloseKey(p_lngRemoteHandle)
   End If
   
   ' ------------------------------------------
   ' Always close the key you opened
   ' ------------------------------------------
   p_lngRtn = RegCloseKey(p_lngKeyHandle)

   If p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_NO_CLOSE_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegisterEventSource"
      m_strErrDesc = "Could not CLOSE the key, " & _
                     BASE_DIR & ".  The error was: " & _
                     ReturnApiErrString(p_lngRtn)
   End If

End Function

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Sub RegistryEntry
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 11:55:54
' Inputs       : xi_enmMsgType:Long -
' Outputs      : N/A
' Description  : Get the registry entry for EventMessageFile or
'              :     PamameterMessageLog entry --
'              : Have to set the Log Type and SourceName before
'              :     calling this Property
' *******************************************************
Public Sub RegistryEntry(ByVal xi_enmMsgType As enmMessageLookup)
On Error Resume Next                   ' Don't accept an error here
   Dim p_lngKeyType                    As Long
   Dim p_lngRtn                        As Long
   Dim p_lngOpenedKey                  As Long
   Dim p_lngLenReturnedData            As Long
   Dim p_strReturnString               As String
   Dim p_strValue                      As String
   Dim p_strErrorStr                   As String
   Dim p_strKey                        As String
   Dim p_strValueName                  As String

   ' -----------------------------------------------------
   ' Set the value for the Value Name
   ' -----------------------------------------------------
   If xi_enmMsgType = ParameterMessage Then
      p_strValueName = "ParameterMessageFile"
   ElseIf xi_enmMsgType = CategoryMessage Then
      p_strValueName = "CategoryMessageFile"
   Else
      p_strValueName = "EventMessageFile"
   End If

   ' -----------------------------------------------------
   ' Check to make sure that the 2 variables have been set
   ' -----------------------------------------------------
   If Len(Trim$(m_strLogType)) = 0 Or _
      CStr(m_strLogType) = CStr(&HFFFFFFFF) Then
      m_lngErrNumber = (ERR_LOGTYPE_NOT_SET + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegistryEntry"
      m_strErrDesc = "The Log Type has not been set."
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
      
   ElseIf Len(Trim$(m_strSourceName)) = 0 Then
      m_lngErrNumber = (ERR_SOURCENAME_NOT_SET + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegistryEntry"
      m_strErrDesc = "The Source Name has not been set."
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
      
   End If

   ' --------------------------------------------------
   ' Open key
   ' --------------------------------------------------
   p_strKey = "SYSTEM\CurrentControlSet\Services\EventLog\" _
              & m_strLogType & "\" & m_strSourceName
   p_lngRtn = RegOpenKeyEx(HKEY_LOCAL_MACHINE, p_strKey, 0&, KEY_READ, _
         p_lngOpenedKey)
   
   If p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_FAILED_OPEN_REGISTRY_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegistryEntry"
      m_strErrDesc = "Error opening Registry Key (RegOpenKeyEx), " & "'" & p_strKey & "'" _
                        & "." & vbCrLf & "The error was: " & ReturnApiErrString(p_lngRtn)
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
   End If

   ' --------------------------------------------------
   ' Set up buffer for data to be returned in.
   ' --------------------------------------------------
   p_lngLenReturnedData = 255
   p_strReturnString = Space$(p_lngLenReturnedData)

   ' --------------------------------------------------
   ' Read key -- try again if buffer isn't big enough
   ' --------------------------------------------------
ValueTryAgain:
   p_lngRtn = RegQueryValueEx(p_lngOpenedKey, p_strValueName, ByVal 0&, _
         p_lngKeyType, ByVal p_strReturnString, p_lngLenReturnedData)

   If p_lngRtn = ERROR_MORE_DATA Then
      p_strReturnString = Space$(p_lngLenReturnedData)
      GoTo ValueTryAgain
   
   ElseIf p_lngRtn <> ERROR_SUCCESS Then
      m_lngErrNumber = (ERR_FAILED_READ_REGISTRY_KEY + vbObjectError)
      m_strErrSource = REG_SOURCENAME & "RegistryEntry"
      m_strErrDesc = "Error querying the Registry Entry value (RegQueryValueEx), " _
                           & "'" & p_strKey & "'" & " for the Value Name, " _
                           & "'" & p_strValueName & "'." & vbCrLf & "The error was: " _
                           & ReturnApiErrString(p_lngRtn)
      On Error GoTo 0
      Err.Raise m_lngErrNumber, m_strErrSource, m_strErrDesc
      
   Else
      m_strRegistryEntry = Left$(p_strReturnString, p_lngLenReturnedData - 1)
   
   End If

Cleanup:
   ' --------------------------------------------------
   ' Always close opened keys!
   ' --------------------------------------------------
   p_lngRtn = RegCloseKey(p_lngOpenedKey)

End Sub

' ***********************************************************************
' ***********************************************************************
' ****                     Read/Write Properties                     ****
' ***********************************************************************
' ***********************************************************************

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Let LogType
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 11:53:37
' Inputs       : ByVal xi_strLogType:String -
' Outputs      : N/A
' Description  : Get/Let the log type
' *******************************************************
Public Property Let LogType(ByVal xi_strLogType As String)
   m_strLogType = xi_strLogType
End Property

Public Property Get LogType() As String
   LogType = m_strLogType
End Property

' *******************************************************
' Routine Name : (PUBLIC in CLASS) Property Let SourceName
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 11:53:12
' Inputs       : ByVal xi_strSourceName:String -
' Outputs      : N/A
' Description  : Get/Let the Source name
' *******************************************************
Public Property Let SourceName(ByVal xi_strSourceName As String)
   m_strSourceName = xi_strSourceName
End Property

Public Property Get SourceName() As String
   SourceName = m_strSourceName
End Property

' ***********************************************************************
' ***********************************************************************
' ****                        READ-ONLY Properties                   ****
' ***********************************************************************
' ***********************************************************************

' -------------------------------------------------
' Get the Value string for the registry value entry
' -------------------------------------------------
Public Property Get RegistryEntryText() As String
   RegistryEntryText = m_strRegistryEntry
End Property

' ***********************************************************************
' ***********************************************************************
' ****                           CLASS Properties                    ****
' ***********************************************************************
' ***********************************************************************

' *******************************************************
' Routine Name : (PRIVATE in CLASS) Sub Class_Initialize
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 11:48:30
' Inputs       : N/A
' Outputs      : N/A
' Description  : Set 2 strings to zero-lengths on initialization
' *******************************************************
Private Sub Class_Initialize()
   m_strLogType = vbNullString
   m_strSourceName = vbNullString
End Sub
