Attribute VB_Name = "modSID"
Option Explicit

Private Const SID_REVISION             As Long = 1&   ' Current revision level
Private Const SID_MAX_SUB_AUTHORITIES  As Long = 15&

Private Type SID_IDENTIFIER_AUTHORITY
  Value(5)                             As Byte
End Type

Private Declare Function IsValidSid _
   Lib "advapi32" _
   (ByVal pSid As Any) As Long
Private Declare Function GetSidIdentifierAuthority _
   Lib "advapi32" _
   (pSid As Any) As Long
Private Declare Function GetSidSubAuthorityCount _
   Lib "advapi32" _
   (pSid As Any) As Long
Private Declare Function GetSidSubAuthority _
   Lib "advapi32" _
   (pSid As Any, _
    ByVal nSubAuthority As Long) As Long
Private Declare Function LookupAccountSid _
   Lib "advapi32.dll" Alias "LookupAccountSidA" _
   (ByVal lpSystemName As String, _
    ByVal Sid As Long, _
    ByVal Name As String, _
    cbName As Long, _
    ByVal ReferencedDomainName As String, _
    cbReferencedDomainName As Long, _
    peUse As Long) As Long
Private Declare Function LookupAccountName _
   Lib "advapi32.dll" Alias "LookupAccountNameA" _
   (ByVal lpSystemName As String, _
    ByVal lpAccountName As String, _
    Sid As Any, _
    cbSID As Long, _
    ReferencedDomainName As Any, _
    cbReferencedDomainName As Long, _
    peUse As Integer) As Long

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function FormatHex
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 15:34:55
' Inputs       : ByVal xi_lngValue:Long    -
'              : ByVal xi_lngNumChars:Long -
' Outputs      : STRING:
' Description  : Used to format the SidIdentifierAuthority
' *******************************************************
Public Function FormatHex(ByVal xi_lngValue As Long, _
                          ByVal xi_lngNumChars As Long) As String
   FormatHex = String$(xi_lngNumChars - Len(Hex(xi_lngValue)), "0") & Hex(xi_lngValue)
End Function

' *******************************************************
' Routine Name : (PUBLIC in MODULE) Function GetTextualSid
' Written By   : L.J  Johnson
' Programmer   : L.J  Johnson [Slightly Tilted Software]
' Date Writen  : 05/29/2001 -- 15:26:56
' Inputs       : ByVal xi_lngPtrSID:Long -
' Outputs      : STRING:
' Description  : Thanks to Brad Martinez for the code, just
'              :     had to clear up a problem with signed longs
'              : Given a pointer to a SID, return a texual
'              :     representation of that SID
' *******************************************************
Public Function GetTextualSid(ByVal xi_lngPtrSID As Long) As String
   Dim p_typSecurityID                 As SID_IDENTIFIER_AUTHORITY
   Dim p_lngRtn                        As Long
   Dim p_lngNumSubAuthorities          As Long
   Dim p_lngLoop                       As Long
   Dim p_lngPtrSubAuthorities          As Long
   Dim p_dblBigNum                     As Double
   Dim p_strSID                        As String
   Const OFFSET_4                      As Double = 4294967296#
   
   ' ------------------------------------------
   ' test if Sid passed in is valid
   ' ------------------------------------------
   If IsValidSid(ByVal xi_lngPtrSID) Then

      ' ---------------------------------------
      ' Get the SID's identifier authority value
      '    (if the function fails, the return
      '     value is undefined)
      ' ---------------------------------------
      p_lngRtn = GetSidIdentifierAuthority(ByVal xi_lngPtrSID)
    
      If p_lngRtn And (Err.LastDllError = 0) Then
         MoveMem p_typSecurityID, ByVal p_lngRtn, Len(p_typSecurityID)
   
         ' ------------------------------------
         ' prepare S-SID_REVISION-
         ' ------------------------------------
         p_strSID = "S-" & SID_REVISION & "-"
     
         ' ------------------------------------
         ' prepare SidIdentifierAuthority
         ' ------------------------------------
         If ((p_typSecurityID.Value(0) <> 0) Or (p_typSecurityID.Value(1) <> 0)) Then
            p_strSID = p_strSID & "0x" & _
                       FormatHex((p_typSecurityID.Value(0)), 2) & _
                       FormatHex((p_typSecurityID.Value(1)), 2) & _
                       FormatHex((p_typSecurityID.Value(2)), 2) & _
                       FormatHex((p_typSecurityID.Value(3)), 2) & _
                       FormatHex((p_typSecurityID.Value(4)), 2) & _
                       FormatHex((p_typSecurityID.Value(5)), 2)
         Else
            p_strSID = p_strSID & _
                       (p_typSecurityID.Value(5) + _
                       (p_typSecurityID.Value(4) * 2 ^ 8) + _
                       (p_typSecurityID.Value(3) * 2 ^ 16) + _
                       (p_typSecurityID.Value(2) * 2 ^ 24))
         End If
       
         ' ------------------------------------
         ' obtain the SID's sub-authority count
         '     and add each to the string
         ' ------------------------------------
         p_lngRtn = GetSidSubAuthorityCount(ByVal xi_lngPtrSID)
       
         If p_lngRtn Then
            MoveMem p_lngNumSubAuthorities, ByVal p_lngRtn, 4
          
            For p_lngLoop = 0 To p_lngNumSubAuthorities - 1
               p_lngRtn = GetSidSubAuthority(ByVal xi_lngPtrSID, p_lngLoop)
             
               If p_lngRtn <> 0 Then
                  MoveMem p_lngPtrSubAuthorities, ByVal p_lngRtn, 4
                  
                  ' ---------------------------
                  ' Have to handle the #@$&#@(
                  '     signed longs
                  ' ---------------------------
                  If p_lngPtrSubAuthorities < 0 Then
                     p_dblBigNum = p_lngPtrSubAuthorities + OFFSET_4
                     p_strSID = p_strSID & "-" & p_dblBigNum
                  Else
                     p_strSID = p_strSID & "-" & p_lngPtrSubAuthorities
                  End If
               End If
            Next
       
         End If
      
      End If
   
   End If
  
   GetTextualSid = p_strSID
  
End Function
